/**************************************************************************
 * Module name:   callback.c						  *
 * Project:       xps (Process State Programm for X			  *
 * written by:    Thomas Breitfeld (breitf@infotech.tu-chemnitz.de)	  *
 * last change:   11. 4. 1994                                             *
 *									  *
 * Copyright (c) 1994 Technical University of Chemnitz-Zwickau		  *
 *		      Department of Electrical Engineering		  *
 *		      Division of Data and Communication Technologies	  *
 *									  *
 * ---------------------------------------------------------------------- *
 * description:   This module contains all event functions of the 	  *
 *	          user interface, like buttons, menus etc		  *
 * ---------------------------------------------------------------------- *
 * entry points:  button_info						  *
 *		  menu_customize_proc                                     *
 *		  button_cust_def_proc                                    *
 *		  button_cust_apply_proc                                  *
 *		  button_filter_proc                                      *
 *		  button_filter_def_proc                                  *
 *		  button_filter_apply_proc                                *
 *		  button_prop_proc                                        *
 *		  button_extended_proc					  *
 *		  button_resource_proc					  *
 *		  menu_action_proc                                        *
 *		  menu_signal_proc					  *
 *		  menu_list_proc   	                                  *
 *		  menu_tree_proc					  *
 *		  menu_child_proc					  *
 * 		  menu_all_proc						  * 
 *		  canvas_event_proc					  *
 *		  panel_filter_user_proc				  *
 *		  panel_filter_tty_proc					  *
 *		  button_control_proc					  *
 *                slider_control_proc					  *
 *                fb_button_control_proc				  *
 *                replay_button_control_proc				  *
 *                ff_button_control_proc				  *
 *                goahead_button_control_proc				  *
 *                stop_button_control_proc				  *
 **************************************************************************/

#include<signal.h>
#include<errno.h>
#include<sys/types.h>

#include"xpis.h"
#include"callback.h"
#include"filter.h"
#include"paint.h"
#include"misc.h"
#include"xpsgc.h"
#include"control.h"
#include"keys.h"
#include"prop.h"

/*
 * PRIVATE function prototypes
 */
PRIVATE void write_right_footer(void);
PRIVATE void write_left_footer(int text);
PRIVATE void write_signal_footer(int sig, int pid);
PRIVATE bool meta = NO;
PRIVATE bool sleft = NO;
PRIVATE bool animate = YES;


/* -------------------------------------------------------------------
 * function: button_info
 * -------------------------------------------------------------------
 */
PUBLIC void button_info(Panel_item quit_item,Event *event)
{
   int x, y;
   Frame base_frame = xv_get(quit_item, XV_KEY_DATA, BASE_FRAME_KEY);

   x = event_x(event), y = event_y(event);
   notice_prompt(base_frame, event,
	NOTICE_FOCUS_XY,	x, y,
        NOTICE_MESSAGE_STRINGS,
#ifdef linux
 "Xps 1.0 for Linux",
#else
 "Xps 1.0 for SunOS 4.x.",
#endif
		"\nunder GPL (c) 1994",
	        "Thomas Breitfeld",
		"TU Chemnitz, Germany",
		"Institute of Information Technology\n",
	        "suggestions and comments to:",
		"xps@infotech.tu-chemnitz.de\n",
		"August 28 1994", NULL,
	NOTICE_BUTTON_YES,	"Done",
	NOTICE_NO_BEEPING,	TRUE,
	NULL);
}


/* -------------------------------------------------------------------
 * function: menu_customize_proc
 * -------------------------------------------------------------------
 */
PUBLIC void menu_customize_proc(Menu view_menu,Menu_item cust_item)
{
   Frame base_frame = xv_get(cust_item,XV_KEY_DATA,BASE_FRAME_KEY);
   Frame cust_frame = xv_get(cust_item,XV_KEY_DATA,CUST_FRAME_KEY);

   if(xv_get(cust_frame,XV_KEY_DATA,CUST_SHOWED_KEY) == FALSE) {
      xv_set(cust_frame,
	     XV_Y,		(int)xv_get(base_frame,XV_Y) - 26,
	     XV_X,		(int)xv_get(base_frame,XV_X) +
				(int)xv_get(base_frame,XV_WIDTH) + 5,
	     XV_KEY_DATA,	CUST_SHOWED_KEY,	TRUE,
	     NULL);
   }

   xv_set(cust_frame,XV_SHOW,TRUE,NULL);
}


/* -------------------------------------------------------------------
 * function: button_cust_def_proc
 * -------------------------------------------------------------------
 */
PUBLIC void button_cust_def_proc(Panel_item def_item,Event *event)
{
   Panel_item pitem;

   pitem = xv_get(def_item, XV_KEY_DATA, ICON_CUST_KEY);
   xv_set(pitem, PANEL_VALUE, I_REGULAR, NULL);
   pitem = xv_get(def_item, XV_KEY_DATA, FONT_CUST_KEY);
   xv_set(pitem, PANEL_VALUE, F_MEDIUM, NULL);
   pitem = xv_get(def_item, XV_KEY_DATA, ANIMATE_CUST_KEY);
   xv_set(pitem, PANEL_VALUE, 0, NULL);

}


/* -------------------------------------------------------------------
 * function: button_cust_apply_proc
 * -------------------------------------------------------------------
 */
PUBLIC void button_cust_apply_proc(Panel_item app_item,Event *event)
{
   Panel_item pitem;
   int iconstyle, fontsize;

   pitem = xv_get(app_item, XV_KEY_DATA, ICON_CUST_KEY);
   iconstyle = xv_get(pitem, PANEL_VALUE);
   pitem = xv_get(app_item, XV_KEY_DATA, FONT_CUST_KEY);
   fontsize = xv_get(pitem, PANEL_VALUE);
   pitem = xv_get(app_item, XV_KEY_DATA, ANIMATE_CUST_KEY);
   animate = !xv_get(pitem, PANEL_VALUE);

   paint(setgc(iconstyle,fontsize,CHANGE_NOT));
}


/* -------------------------------------------------------------------
 * function: button_filter_proc
 * -------------------------------------------------------------------
 */
PUBLIC void button_filter_proc(Panel_item filter_item,Event *event)
{
   Frame base_frame = xv_get(filter_item,XV_KEY_DATA,BASE_FRAME_KEY);
   Frame filter_frame = xv_get(filter_item,XV_KEY_DATA,FILTER_FRAME_KEY);

   if(xv_get(filter_frame,XV_KEY_DATA,FILTER_SHOWED_KEY) == FALSE) {
      xv_set(filter_frame,
	     XV_Y,		(int)xv_get(base_frame,XV_Y) - 26,
	     XV_X,		(int)xv_get(base_frame,XV_X) +
				(int)xv_get(base_frame,XV_WIDTH) + 5,
	     XV_KEY_DATA,	FILTER_SHOWED_KEY,	TRUE,
	     NULL);
   }

   xv_set(filter_frame,XV_SHOW,TRUE,NULL);
}


/* -------------------------------------------------------------------
 * function: button_filter_def_proc
 * -------------------------------------------------------------------
 */
PUBLIC void button_filter_def_proc(Panel_item f_def_item,Event *event)
{
}


/* -------------------------------------------------------------------
 * function: button_filter_apply_proc
 * -------------------------------------------------------------------
 */
PUBLIC void button_filter_apply_proc(Panel_item f_app_item,Event *event)
{
   Panel_item pitem;
   char *p_text;
   int i;
   int uids[NUMBER_F_UIDS];
   char *ttys[NUMBER_F_TTYS];
   char fttytext[NUMBER_F_TTYS][TTY_SIGN_LENGTH+1];
   States states;

 /*init ttys pointers
  */
   for(i=0;i<NUMBER_F_TTYS;i++)
      ttys[i] = fttytext[i];

 /*get user filter
  */
   pitem = xv_get(f_app_item,XV_KEY_DATA,USER_FILTER_KEY);
   if(xv_get(pitem,PANEL_VALUE) == U_ONLY) {
      pitem = xv_get(f_app_item,XV_KEY_DATA,USER_TEXT_FILTER_KEY);
      p_text = (char *)xv_get(pitem,PANEL_VALUE);
      if(text_to_fuids(p_text,uids) < 0)
	 *uids = END_F_UIDS_LIST;

   }
   else
      *uids = END_F_UIDS_LIST;

 /*get tty filter
  */
   pitem = xv_get(f_app_item,XV_KEY_DATA,TTY_FILTER_KEY);
   if(xv_get(pitem,PANEL_VALUE) == TTY_WITHOUT) {
      pitem = xv_get(f_app_item,XV_KEY_DATA,TTY_TEXT_FILTER_KEY);
      p_text = (char *)xv_get(pitem,PANEL_VALUE);
      if(text_to_fttys(p_text,ttys,TTY_SIGN_LENGTH) < 0)
	 *ttys = NULL;
   }
   else
      *ttys = NULL;

 /*get state filter
  */
   pitem = xv_get(f_app_item,XV_KEY_DATA,STATE_FILTER_KEY);
   if(xv_get(pitem,PANEL_VALUE) == STATE_ONLY) {
      pitem = xv_get(f_app_item,XV_KEY_DATA,STATE_TOGGLE_FILTER_KEY);
      i = xv_get(pitem,PANEL_VALUE);
      if(pval_to_fstate(i,&states) < 0) {
	 states.letter_1st[0] = 0;
	 states.letter_2cd[0] = 0;
      }
   }
   else {
      states.letter_1st[0] = 0;
      states.letter_2cd[0] = 0;
   }	 

 /*set new filter parameters and actualize painting
  */
   initfp(uids,ttys,&states);
   play_current_psi();
}


/* -------------------------------------------------------------------
 * function: panel_filter_user_proc
 * -------------------------------------------------------------------
 */
PUBLIC void panel_filter_user_proc(Panel_item f_user_item,Event *event)
{
   Panel_item user_text;

   user_text = xv_get(f_user_item,XV_KEY_DATA,USER_TEXT_FILTER_KEY);
   if(xv_get(f_user_item,PANEL_VALUE) == U_ALL)
      xv_set(user_text,PANEL_INACTIVE,TRUE,NULL);
   else
      xv_set(user_text,PANEL_INACTIVE,FALSE,NULL);
}


/* -------------------------------------------------------------------
 * function: panel_filter_tty_proc
 * -------------------------------------------------------------------
 */
PUBLIC void panel_filter_tty_proc(Panel_item f_tty_item,Event *event)
{
   Panel_item tty_text;

   tty_text = xv_get(f_tty_item,XV_KEY_DATA,TTY_TEXT_FILTER_KEY);
   if(xv_get(f_tty_item,PANEL_VALUE) == TTY_ALL)
      xv_set(tty_text,PANEL_INACTIVE,TRUE,NULL);
   else
      xv_set(tty_text,PANEL_INACTIVE,FALSE,NULL);
}


/* -------------------------------------------------------------------
 * function: panel_filter_state_proc
 * -------------------------------------------------------------------
 */
PUBLIC void panel_filter_state_proc(Panel_item f_state_item,Event *event)
{
   Panel_item state_choice;

   state_choice = xv_get(f_state_item,XV_KEY_DATA,STATE_TOGGLE_FILTER_KEY);
   if(xv_get(f_state_item,PANEL_VALUE) == STATE_ALL)
      xv_set(state_choice,PANEL_INACTIVE,TRUE,NULL);
   else
      xv_set(state_choice,PANEL_INACTIVE,FALSE,NULL);
}


/* -------------------------------------------------------------------
 * function: button_prop_proc
 * -------------------------------------------------------------------
 */
PUBLIC void button_prop_proc(Panel_item prop_item,Event *event)
{
   Frame base_frame = xv_get(prop_item,XV_KEY_DATA,BASE_FRAME_KEY);
   Frame prop_frame = xv_get(prop_item,XV_KEY_DATA,PROP_FRAME_KEY);

   if(xv_get(prop_frame,XV_KEY_DATA,PROP_SHOWED_KEY) == FALSE) {
      xv_set(prop_frame,
	     XV_Y,		(int)xv_get(base_frame,XV_Y) - 26,
	     XV_X,		(int)xv_get(base_frame,XV_X) +
				(int)xv_get(base_frame,XV_WIDTH) + 5,
	     XV_KEY_DATA,	PROP_SHOWED_KEY,	TRUE,
	     NULL);
   }

   xv_set(prop_frame,XV_SHOW,TRUE,NULL);
   if(set_pitems_s() < 0)
      write_left_footer(3);
}


/* -------------------------------------------------------------------
 * function: button_extended_proc
 * -------------------------------------------------------------------
 */
PUBLIC void button_extended_proc(Panel_item item,Event *event)
{
   Frame base_frame = get_base_frame();
   Frame extprop_frame = xv_get(item,XV_KEY_DATA,EXTPROP_FRAME_KEY);

   if(xv_get(extprop_frame,XV_KEY_DATA,EXTPROP_SHOWED_KEY) == FALSE) {
      xv_set(extprop_frame,
	     XV_Y,		(int)xv_get(base_frame,XV_Y) +
		  (int)xv_get(xv_get(base_frame,XV_KEY_DATA,PROP_FRAME_KEY),
				XV_HEIGHT),
	     XV_X,		(int)xv_get(base_frame,XV_X) +
				(int)xv_get(base_frame,XV_WIDTH) + 5,
	     XV_KEY_DATA,	EXTPROP_SHOWED_KEY,	TRUE,
	     NULL);
   }

   xv_set(extprop_frame,XV_SHOW,TRUE,NULL);
   if(set_pitems_s() < 0)
      write_left_footer(3);
}


/* -------------------------------------------------------------------
 * function: button_resource_proc
 * -------------------------------------------------------------------
 */
PUBLIC void button_resource_proc(Panel_item item,Event *event)
{
   Frame base_frame = get_base_frame();
   Frame resprop_frame = xv_get(item,XV_KEY_DATA,RESPROP_FRAME_KEY);

   if(xv_get(resprop_frame,XV_KEY_DATA,RESPROP_SHOWED_KEY) == FALSE) {
      xv_set(resprop_frame,
	     XV_Y,		(int)xv_get(base_frame,XV_Y) +
		  (int)xv_get(xv_get(base_frame,XV_KEY_DATA,PROP_FRAME_KEY),
				XV_HEIGHT),
	     XV_X,		(int)xv_get(base_frame,XV_X) +
				(int)xv_get(base_frame,XV_WIDTH) + 5,
	     XV_KEY_DATA,	RESPROP_SHOWED_KEY,	TRUE,
	     NULL);
   }

   xv_set(resprop_frame,XV_SHOW,TRUE,NULL);
   if(set_pitems_s() < 0)
      write_left_footer(3);
}


/* -------------------------------------------------------------------
 * function: menu_action_proc
 * -------------------------------------------------------------------
 */
PUBLIC void menu_action_proc(Menu action_menu,Menu_item m_item)
{
   int pid;
   int killval;
   ProcessToDraw *proc;
   XpisGC *gc = getgc();

   if((pid = get_select()) >= 0) {
      if((proc = pdt_search(pid)) != NULL) {
         if((proc->pstate->ruid == getuid()) || (getuid() == 0)) {
            switch(xv_get(m_item,MENU_VALUE)) {
               case KILL_SOFT: killval = kill(pid,SIGTERM);
	   		       break;
               case KILL_HARDT:killval = kill(pid,SIGKILL);
			       break;
            }
            if(killval == -1) {
               switch(errno) {
                  case EPERM:
	             write_left_footer(0);
	             break;
	          default:
	             write_left_footer(1);
	             break;
               }
            }
            else {
	       proc->picon.image = gc->images->killer;
	       paint_object(proc->picon,gc);
            }
         } else
	    write_left_footer(0);
      }
      else {
	 clear_select();
	 write_left_footer(2);
      }
   }
   else 
      write_left_footer(2);
}


/* -------------------------------------------------------------------
 * function: menu_signal_proc
 * -------------------------------------------------------------------
 */
PUBLIC void menu_signal_proc(Menu menu,Menu_item m_item)
{
   int i, sig_nr, pid, killval=0;
   char **p, *s = (char *)xv_get(m_item, MENU_STRING);
   char *parent = (char *)xv_get(xv_get(menu, MENU_PARENT), MENU_STRING);
   ProcessToDraw *proc;

   if((pid = get_select()) >= 0) {
      if((proc = pdt_search(pid)) != NULL) {
         if((proc->pstate->ruid == getuid()) || (getuid() == 0) \
            || ((meta == YES) && (sleft == YES))) {

            /* get signal number */
            for(p=signals, i=0; *p!=NULL; p++, i++) {
               if(!strcmp(s,*p)) {
	          sig_nr = sigindex_2_nr(i);
	          break;
               }
            }

	    /* send signal to pid or pgrp*/
	    if(!strcmp(parent,"Signal to Process")) {
	       killval = kill(pid, sig_nr);
	       write_signal_footer(sig_nr, pid);
            } else if(!strcmp(parent,"Signal to Group")) {
	       killval = kill(-proc->pstate->pgrp, sig_nr);
	       write_signal_footer(sig_nr, -proc->pstate->pgrp);
            }

	    /* check errors */
            if(killval == -1) {
               switch(errno) {
                  case EPERM:
	             write_left_footer(7);
	             break;
	          default:
	             write_left_footer(1);
	             break;
               }
            }
	    meta = NO; sleft = NO;
         }
         else
	    write_left_footer(7);
      }
      else {
	 clear_select();
	 write_left_footer(6);
      }
   }
   else
      write_left_footer(6);
}


/* -------------------------------------------------------------------
 * function: menu_list_proc
 * -------------------------------------------------------------------
 */
PUBLIC void menu_list_proc(Menu view_menu,Menu_item list_item)
{
   setgc(CHANGE_NOT, CHANGE_NOT, NO);
   paint(getgc());
   write_right_footer();
}


/* -------------------------------------------------------------------
 * function: menu_tree_proc
 * -------------------------------------------------------------------
 */
PUBLIC void menu_tree_proc(Menu view_menu,Menu_item tree_item)
{
   setgc(CHANGE_NOT, CHANGE_NOT, YES);
   paint(getgc());
   write_right_footer();
}


/* -------------------------------------------------------------------
 * function: menu_child_proc
 * -------------------------------------------------------------------
 */
PUBLIC void menu_child_proc(Menu view_menu,Menu_item child_item)
{
   int pid;

   if((pid = get_select()) < 0) {
      write_left_footer(4);
   }
   else {
      set_childs(pid);
      play_current_psi();
      write_right_footer();
   }
}


/* -------------------------------------------------------------------
 * function: menu_all_proc
 * -------------------------------------------------------------------
 */
PUBLIC void menu_all_proc(Menu view_menu,Menu_item all_item)
{
   if(clear_childs() < 0)
      write_left_footer(5);
   else {
      play_current_psi();
      write_right_footer();
   }
}


/* -------------------------------------------------------------------
 * function: button_control_proc
 * -------------------------------------------------------------------
 */
PUBLIC void button_control_proc(Panel_item control_item,Event *event)
{
   Frame base_frame = xv_get(control_item,XV_KEY_DATA,BASE_FRAME_KEY);
   Frame control_frame = xv_get(control_item,XV_KEY_DATA,CONTROL_FRAME_KEY);
       
   if(xv_get(control_frame,XV_KEY_DATA,CONTROL_SHOWED_KEY) == FALSE) {
      xv_set(control_frame,
	     XV_Y,		(int)xv_get(base_frame,XV_Y) - 26,
	     XV_X,		(int)xv_get(base_frame,XV_X) +
				(int)xv_get(base_frame,XV_WIDTH) + 5,
	     XV_KEY_DATA,	CONTROL_SHOWED_KEY,	TRUE,
	     NULL);
   }

   xv_set(control_frame,XV_SHOW,TRUE,NULL);
}


/* -------------------------------------------------------------------
 * function: slider_control_proc
 * -------------------------------------------------------------------
 */
PUBLIC void slider_control_proc(Panel_item item, int value)
{
   static int last_value = 0;

   if(value != last_value) {
      last_value = value;
      play(ACT_N, -value, NO);
   }
}


/* -------------------------------------------------------------------
 * function: fb_button_control_proc
 * -------------------------------------------------------------------
 */
PUBLIC void fb_button_control_proc(Panel_item item, Event *event)
{
   play(DECREMENT,0,YES);
}


/* -------------------------------------------------------------------
 * function: replay_button_control_proc
 * -------------------------------------------------------------------
 */
PUBLIC void replay_button_control_proc(Panel_item item, Event *event)
{
   Panel_item slider = xv_get(item,XV_KEY_DATA,SLIDER_CONTROL_KEY);
   Panel_item replay_timer = xv_get(item,XV_KEY_DATA,REPLAY_TIMER_KEY);

   if(xv_get(slider,PANEL_VALUE) == 0)
      play(LAST,0,YES);

   adjust_timer(xv_get(replay_timer, PANEL_VALUE),0,REPLAY);

   /* change myself to stop button */
   xv_set(item,
	     PANEL_LABEL_IMAGE, xv_get(item,XV_KEY_DATA,STOP_IMAGE_KEY),
	     PANEL_NOTIFY_PROC, stop_button_control_proc,
	     NULL);

}


/* -------------------------------------------------------------------
 * function: ff_button_control_proc
 * -------------------------------------------------------------------
 */
PUBLIC void ff_button_control_proc(Panel_item item, Event *event)
{
   play(INCREMENT,0,YES);
}


/* -------------------------------------------------------------------
 * function: stop_button_control_proc
 * -------------------------------------------------------------------
 */
PUBLIC void stop_button_control_proc(Panel_item item, Event *event)
{
   if(stop_timer()->cb_func == RUN_PS) {
      /*set slider and control buttons active*/
      xv_set(xv_get(item,XV_KEY_DATA,SLIDER_CONTROL_KEY),
	     PANEL_INACTIVE,	FALSE,
	     PANEL_MIN_VALUE,	-(get_track_entries() - 1),
	     PANEL_VALUE,	0,	NULL);
      xv_set(xv_get(item,XV_KEY_DATA,FB_CONTROL_KEY),
	     PANEL_INACTIVE,	FALSE,	NULL);
   }
   /*change myself to replay button */
   xv_set(item,
	     PANEL_LABEL_IMAGE, xv_get(item,XV_KEY_DATA,REPLAY_IMAGE_KEY),
	     PANEL_NOTIFY_PROC, replay_button_control_proc,
	     NULL);
}


/* -------------------------------------------------------------------
 * function: goahead_button_control_proc
 * -------------------------------------------------------------------
 */
PUBLIC void goahead_button_control_proc(Panel_item item,Event *event)
{
   Panel_item sec_item = xv_get(item,XV_KEY_DATA,XPS_TIMER_SEC_KEY);
   Panel_item sec10_item = xv_get(item,XV_KEY_DATA,XPS_TIMER_10SEC_KEY);
   Panel_item nr_images =xv_get(item,XV_KEY_DATA,NR_IMAGES_KEY);

   /* reinit recorder track */
   if(change_track_length(xv_get(nr_images, PANEL_VALUE)) < 0)
      puterr("goahead_button_proc: change_track_length failed - no memory\n");

   /* set timer */
   if(adjust_timer(xv_get(sec_item, PANEL_VALUE),
		   xv_get(sec10_item, PANEL_VALUE),
		   RUN_PS)->cb_func != RUN_PS) { 
      /* move slider to 0*/
      play(ACT_N,0,YES);
      /* deactivate slider and control buttons*/
      xv_set(xv_get(item,XV_KEY_DATA,SLIDER_CONTROL_KEY),
	     PANEL_INACTIVE,	TRUE,	NULL);
      xv_set(xv_get(item,XV_KEY_DATA,FB_CONTROL_KEY),
	     PANEL_INACTIVE,	TRUE,	NULL);
      xv_set(xv_get(item,XV_KEY_DATA,FF_CONTROL_KEY),
	     PANEL_INACTIVE,	TRUE,	NULL);
      /* change replay button to stop button */
      xv_set(xv_get(item,XV_KEY_DATA,REPLAY_CONTROL_KEY),
	     PANEL_LABEL_IMAGE, xv_get(item,XV_KEY_DATA,STOP_IMAGE_KEY),
	     PANEL_NOTIFY_PROC, stop_button_control_proc,
	     NULL);
   }
}


/* -------------------------------------------------------------------
 * function: canvas_event_proc
 * -------------------------------------------------------------------
 */
PUBLIC void canvas_event_proc(Xv_Window window, Event *event)
{
   switch(event_action(event)) {
      case ACTION_SELECT:
	 if(event_is_down(event)) {
	    item_select(event_x(event),event_y(event));
	    /*clear left footer*/
	    xv_set(get_base_frame(),FRAME_LEFT_FOOTER,"",NULL);
	 }
	 break;
      case SHIFT_LEFT:
         if(event_is_down(event))
	    sleft = YES;
	 else
	    sleft = NO;
	 break;
      case SHIFT_LEFTCTRL:
         if(event_is_down(event))
	    meta = YES;
	 else
	    meta = NO;
	 break;
   }
}


/* -------------------------------------------------------------------
 * function: base_frame_event_proc
 * -------------------------------------------------------------------
 */
PUBLIC void base_frame_event_proc(Xv_Window window, Event *event)
{
   static struct timer_state tstate = { 1, 0, RUN_PS };
   struct timer_state *tp;

   switch(event_action(event)) {
      case ACTION_OPEN:
	 adjust_timer(tstate.sec,tstate.sec10,tstate.cb_func);
	 break;
      case ACTION_CLOSE:
	 if(animate) 
	    tp = adjust_timer(0,5,ANIMATE);
	 else
	    tp = stop_timer();
	 tstate = *tp;
	 break;
      case WIN_RESIZE:
	 if(event_xevent(event)->xconfigure.send_event == TRUE) {
	    if(getgc()->tree == NO)
	       paint(getgc());
         }
   }
}


/* ----------------------------------------------------
 * function: write_right_footer
 * ----------------------------------------------------
 */
PRIVATE void write_right_footer(void)
{
   char footer[RIGHT_FOOTER_LENGTH];
   int pid;

   if((pid = get_childs()) < 0) 
      sprintf(footer,"all processes as %s",(getgc()->tree) ? "tree":"list");
   else
      sprintf(footer,"childs of %d as %s",pid,(getgc()->tree) ? "tree":"list");

   xv_set(get_base_frame(),FRAME_RIGHT_FOOTER,footer,NULL);
}

/* ---------------------------------------------------
 * function: write_left_footer
 * --------------------------------------------------
 */
PRIVATE void write_left_footer(int text)
{
   static char *footers[] = {"You have no permission to kill this process!",
			     "Kill error !!!",
			     "Please select a process to kill it!",
			     "Please select an item to show its properties!",
			     "Please select a process to show its childs!",
			     "All processes are allready sightable",
			     "Please select a process to send a signal!",
			     "You have no permission to send this signal!"};
   xv_set(get_base_frame(),FRAME_LEFT_FOOTER,footers[text],NULL);
}

/* ---------------------------------------------------
 * function: write_signal_footer
 * ---------------------------------------------------
 */
PRIVATE void write_signal_footer(int sig, int pid)
{
   char buf[30];

   if((sig <= NR_SIGNALS) && (sig > 0)) {
      sprintf(buf, "%s to %s %d", signals[sig-1], 
	     (pid<0) ? "Group ID" : "Process ID",
	     abs(pid));
      xv_set(get_base_frame(),FRAME_LEFT_FOOTER,buf,NULL);
   } else
      puterr("write_signal_footer: wrong signal number\n");
}
