/*
 * ps-test
 * Th. Breitfeld breitf@infotech.tu-chemnitz.de
 * 10.8.1994
 */

#include <sys/types.h>
#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include <errno.h>
#include <limits.h>
#include <ctype.h>
#include <time.h>
#include <sys/ioctl.h>
#include <linux/sched.h>
#include <linux/ptrace.h>
#include "xpis.h"
#include "ps.h"
#include "psdata.h"
#include "pst.h"

/* linux task table */
PRIVATE off_t _task;
PRIVATE void get_state_string(char *string, struct task_struct *task);


PUBLIC int getpst_init(void)
{
   /* open ps_database */
   if(open_sys(NULL, 0) == -1) {
      perror("cannot open psdatabase");
      return(-1);
   }

   /* get adress of task table */
   _task = k_addr("_task");

   return(NR_TASKS);
}

PUBLIC int getpst(ProcessState *pst)
{
   struct task_struct task;
   struct task_struct *tp[NR_TASKS];
   char stack[PAGE_SIZE];
   reg_t *sp;
   unsigned long proc_jiffies;
   unsigned long sigignore, sigcatch, bit;
   time_t time_now = time((time_t *)0);
   int i, j, pst_entries = 0;

   /* get task table in one swoap */
   if(kmemread(tp, _task, NR_TASKS*sizeof(struct task *)) == -1)
      return(-1); 
   
   jiffies = get_kword(k_addr("_jiffies"));
   for (i = 0; i < NR_TASKS; ++i) {
	if (tp[i]) {
	    kmemread(&task, (unsigned long) tp[i], sizeof(task));
			/* check if valid, proc may have exited */
	    if ((unsigned) task.state > 5 ||
		    (task.pid <= 0 && i != 0) ||
		    !task.kernel_stack_page)
		continue;
	    
	    if(kmemread(&stack, task.kernel_stack_page, PAGE_SIZE)<0)
                perror("kmemread->stack");
	    sp = (reg_t *)&stack;

	    if (task.pid == 0)
	       task.rss = 0;

	    proc_jiffies = jiffies - task.start_time;
	    pst->pid = task.pid;
	    pst->ppid = get_kword((unsigned long) &task.p_pptr->pid);
            pst->sid = task.session;
	    pst->pgrp = task.pgrp;
	    pst->uid = task.euid;
	    pst->gid = task.egid;
	    pst->ruid = task.uid;
	    pst->rgid = task.gid;
            pst->pri = 2 *PZERO-task.counter;
	    pst->nice = PZERO-task.priority;
	    pst->ucputime = task.utime / HZ;
	    pst->scputime = task.stime / HZ;
	    pst->tsize = task.end_code;
	    pst->dsize = task.pid == 0 ? 0 : \
                         (task.brk - task.end_code + 1023);
	    pst->ssize = task.pid == 0 ? 0 : SSIZE(sp) ;
            pst->rss = task.rss * 4;
	    pst->minflt = task.min_flt;
	    pst->majflt = task.maj_flt;
	    pst->nswap = task.swap_cnt;
	    pst->flag = task.flags;
            sigignore = 0, sigcatch = 0, bit = 1;
      	    for(j=0; j<32; j++) {
               switch((int) task.sigaction[j].sa_handler) {
    		  case 1: sigignore |= bit; break;
		  case 0: break;
	  	  default: sigcatch |= bit;
	       }
	       bit <<= 1;
            }
            pst->sigignore = sigignore;
	    pst->sigcatch = sigcatch;
	    pst->sigblock = task.blocked;
	    if(proc_jiffies == 0)
		pst->pctcpu = 50.0;
	    else {
		pst->pctcpu = (float)(task.utime+task.stime)*100/proc_jiffies;
		if(pst->pctcpu >= 100.0)
		   pst->pctcpu = 99.9;
	    }
	    strcpy(pst->wchan, (task.state == TASK_INTERRUPTIBLE ||
			        task.state == TASK_UNINTERRUPTIBLE ) ?
			        wchan(task.tss.ebp, sp, 0) : 0);
	    strcpy(pst->text, task.comm);
	    strcpy(pst->tty, dev_to_tty(TASK_TTY(&task)));
	    get_state_string(pst->status, &task);
	    pst->start = time_now - (proc_jiffies/HZ);
	    
	    pst_entries++;
	    pst++;

        }
    }
   return(pst_entries);
}

PRIVATE void get_state_string(char *string, struct task_struct *task)
{
   char *p;

   p = string;
   if(task->state == TASK_INTERRUPTIBLE \
      && task->counter == task->counter / 2 + task->priority)
      *p++ = 'I';
   else
      *p++ = "RSDZTR"[task->state];
   if(task->rss == 0 && task->state != TASK_ZOMBIE && task->pid !=0)
      *p++ = 'W';
   *p = '\0';
}


   
