/**************************************************************************
 * Module name:   misc.c			                          *
 * Project:       xps (Process State Programm for X			  *
 * written by:    Thomas Breitfeld (breitf@infotech.tu-chemnitz.de)	  *
 * last change:   11. 4. 1994                                             *
 *									  *
 * Copyright (c) 1994 Technical University of Chemnitz-Zwickau		  *
 *		      Department of Electrical Engineering		  *
 *		      Division of Data and Communication Technologies	  *
 *									  *
 * ---------------------------------------------------------------------- *
 * description:   This module contains miscellaneous functions		  *
 *                					                  *
 * ---------------------------------------------------------------------- *
 * entry points:  man_select()                                            *
 *		  man_view_childs()                                       *
 *		  char *itoa()						  *
 *		  lfind() for linux only				  *
 **************************************************************************/

#include<stdlib.h>
#include"xpis.h"
#include"misc.h"

/* ---------------------------------------------------------------------
 * function: man_select (manage selected process)
 *           This function keeps all selected processes and gives you
 *	     the possibility to get and set the selections
 * input:    pid -> id of process to store
 *                  if pid = -1 no new pid will be stored
 *		    if pid = -2 man_select will clear all selections
 * return:   id of process stored in man_select before the function was
 *           called or a value below zero to indicate that no
 *           process is selected
 * ---------------------------------------------------------------------
 */
PUBLIC int man_select(int pid)
{
   static int selected_pid = -1;  /* -1 means no process is selected */
   int latch;

   switch(pid) {
      case -1:
	 return(selected_pid);
      case -2:
	 latch = selected_pid;
	 selected_pid = -1;
	 return(latch);
      default:
         latch = selected_pid;
         selected_pid = pid;
         return(latch);
   }
}


/* --------------------------------------------------------------------
 * function: man_view_childs (manage process in view-child-mode)
 *           stores and handels the process from which all childes
 *           will be drawn into the canvas
 * input:    pid -> pid to store
 *                  if pid = -1 no new pid will be stored
 *		    if pid = -2 stored value will be cleared
 * return:   id of stored process or a value below zero, if
 *           no process is stored.
 * --------------------------------------------------------------------
 */
PUBLIC int man_view_childs(int pid)
{
   static int view_childs_of = -1;  /* -1 means no process is stored */
   int latch;

   switch(pid) {
      case -1:
	 return(view_childs_of);
      case -2:
	 latch = view_childs_of;
	 view_childs_of = -1;
	 return(latch);
      default:
         latch = view_childs_of;
         view_childs_of = pid;
         return(latch);
   }
}


/* ---------------------------------------------------------------------
 *  function: itoa
 *             convert integer (decimal) to ascii string.
 * ---------------------------------------------------------------------
 */
PUBLIC char *itoa(int value)
{
    static char buffer[16];
    register char *ptr;
    register int val = abs(value);

    ptr = buffer + sizeof(buffer);
    *--ptr = '\0';
    if(val == 0) {
	*--ptr = '0';
    }
    else while(val != 0) {
	*--ptr = (val % 10) + '0';
	val /= 10;
    }
    if(value < 0) {
       *--ptr = '-';
    }
    return(ptr);
}

#ifdef linux
/* ---------------------------------------------------------------
 * function: lfind
 *	      classical lfind function
 *            because it is not defined in linux I have to 
 *	      rewrite it here
 * _______________________________________________________________
 */
PUBLIC void *lfind(const void *key, const void *base, size_t *num,
                   size_t width, int(*fcmp)(const void *, const void *))
{
   void *b; 
   int i;

   b = (void *)base;
   for(i=0; i<*num; i++) {
      if((fcmp)(key, b) == 0)
         return(b);
      b += width;
   }
   return(NULL);
}
#endif
   
