/**************************************************************************
 * Module name:   prop.c			                          *
 * Project:       xps (Process State Programm for X			  *
 * written by:    Thomas Breitfeld (breitf@infotech.tu-chemnitz.de)	  *
 * last change:   11. 4. 1994                                             *
 *									  *
 * Copyright (c) 1994 Technical University of Chemnitz-Zwickau		  *
 *		      Department of Electrical Engineering		  *
 *		      Division of Data and Communication Technologies	  *
 *									  *
 * ---------------------------------------------------------------------- *
 * description:   contains funktions to display the properties of a	  *
 *		  process in its properties frames			  *
 *                					                  *
 * ---------------------------------------------------------------------- *
 * entry points:  set_prop_items()					  *
 *									  *
 **************************************************************************/

#include<stdio.h>
#include<time.h>
#include<sys/time.h>
#include"xpis.h"
#include"prop.h"
#include"pst.h"
#include"paint.h"
#include"misc.h"
#include"keys.h"


PRIVATE Panel_item p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11,
		   p21, p22, p23, p24, p25, p26, p27, p28, p29, p30,
			p31, p32, p33, p34, p35, p36, p37,
		   p41, p42, p43, p44, p45, p46, p47, p48, p49, p50,
			p51, p52;

PRIVATE Frame prop_frame, extprop_frame, resprop_frame;

PRIVATE char str[PROP_STORED_LENGTH];

/* funktion prototypes */
PRIVATE void prop_items(ProcessToDraw *pd);
PRIVATE void ext_items(ProcessToDraw *pd);
PRIVATE void res_items(ProcessToDraw *pd);
PRIVATE char *sigm_2_str(int sigmask);
PRIVATE char *state_2_str(char *state);


/* -------------------------------------------------------------------
 * function: set_prop_items()
 *           sets all panel items of the process properties frames in
 *           accordance to the XV_SHOW state of the frames and the
 *           selected or given process
 * input:    mode -> 0 (BY_SELECTED) set_prop_items looks by itselfs
 *                                   for the process to show.
 *		     1 (BY_POINTER)  pd is a pointer to the 
 *				     appropriate pdt[] structure
 *           pd  -> pointer to an pdt entry
 *                  if NULL deactivate all items 
 * return:   0 if ok
 *           -1 if no selected process was found
 * -------------------------------------------------------------------
 */
PUBLIC int set_prop_items(int mode, ProcessToDraw *pd)
{
   static allready_called = NO;
   int pid, return_value = 0;
   bool  ext, prop, res;
 
   if(allready_called == NO) {
      prop_frame = xv_get(get_base_frame(), XV_KEY_DATA, PROP_FRAME_KEY);
      extprop_frame = xv_get(get_base_frame(), XV_KEY_DATA,EXTPROP_FRAME_KEY);
      resprop_frame = xv_get(get_base_frame(), XV_KEY_DATA,RESPROP_FRAME_KEY);

      p1  = xv_get(prop_frame, XV_KEY_DATA, PID_PROP_KEY);
      p2  = xv_get(prop_frame, XV_KEY_DATA, COM_PROP_KEY);
      p3  = xv_get(prop_frame, XV_KEY_DATA, UNAME_PROP_KEY);
      p4  = xv_get(prop_frame, XV_KEY_DATA, TTY_PROP_KEY);
      p5  = xv_get(prop_frame, XV_KEY_DATA, STATE_PROP_KEY);
      p6  = xv_get(prop_frame, XV_KEY_DATA, SIZE_PROP_KEY);
      p7  = xv_get(prop_frame, XV_KEY_DATA, RSS_PROP_KEY);
      p8  = xv_get(prop_frame, XV_KEY_DATA, TIME_PROP_KEY);
      p9  = xv_get(prop_frame, XV_KEY_DATA, PCT_PROP_KEY);
      p10 = xv_get(prop_frame, XV_KEY_DATA, NICE_PROP_KEY);
      p11 = xv_get(prop_frame, XV_KEY_DATA, START_PROP_KEY);

      p21 = xv_get(extprop_frame, XV_KEY_DATA, PRI_EXTPROP_KEY);
#ifndef linux
      p22 = xv_get(extprop_frame, XV_KEY_DATA, RESTIME_EXTPROP_KEY);
      p23 = xv_get(extprop_frame, XV_KEY_DATA, SLPTIME_EXTPROP_KEY);
#endif
      p24 = xv_get(extprop_frame, XV_KEY_DATA, SIGIGNORE_EXTPROP_KEY);
      p25 = xv_get(extprop_frame, XV_KEY_DATA, SIGCATCH_EXTPROP_KEY);
      p37 = xv_get(extprop_frame, XV_KEY_DATA, SIGBLOCK_EXTPROP_KEY);
      p26 = xv_get(extprop_frame, XV_KEY_DATA, PPID_EXTPROP_KEY);
      p27 = xv_get(extprop_frame, XV_KEY_DATA, SID_EXTPROP_KEY);
      p28 = xv_get(extprop_frame, XV_KEY_DATA, PGRP_EXTPROP_KEY);
      p29 = xv_get(extprop_frame, XV_KEY_DATA, RUID_EXTPROP_KEY);
      p30 = xv_get(extprop_frame, XV_KEY_DATA, RGID_EXTPROP_KEY);
      p31 = xv_get(extprop_frame, XV_KEY_DATA, EUID_EXTPROP_KEY);
      p32 = xv_get(extprop_frame, XV_KEY_DATA, EGID_EXTPROP_KEY);
      p33 = xv_get(extprop_frame, XV_KEY_DATA, CSIZE_EXTPROP_KEY);
      p34 = xv_get(extprop_frame, XV_KEY_DATA, DSIZE_EXTPROP_KEY);
      p35 = xv_get(extprop_frame, XV_KEY_DATA, SSIZE_EXTPROP_KEY);
      p36 = xv_get(extprop_frame, XV_KEY_DATA, WCH_EXTPROP_KEY);

      p41 = xv_get(resprop_frame, XV_KEY_DATA, UCPUTIME_RESPROP_KEY);
      p42 = xv_get(resprop_frame, XV_KEY_DATA, SCPUTIME_RESPROP_KEY);
      p43 = xv_get(resprop_frame, XV_KEY_DATA, PFIO_RESPROP_KEY);
      p44 = xv_get(resprop_frame, XV_KEY_DATA, PFNIO_RESPROP_KEY);
      p45 = xv_get(resprop_frame, XV_KEY_DATA, SWAPS_RESPROP_KEY);
      p46 = xv_get(resprop_frame, XV_KEY_DATA, BLOCKIN_RESPROP_KEY);
      p47 = xv_get(resprop_frame, XV_KEY_DATA, BLOCKOUT_RESPROP_KEY);
      p48 = xv_get(resprop_frame, XV_KEY_DATA, MSENT_RESPROP_KEY);
      p49 = xv_get(resprop_frame, XV_KEY_DATA, MRECV_RESPROP_KEY);
      p50 = xv_get(resprop_frame, XV_KEY_DATA, SIGNALS_RESPROP_KEY);
      p51 = xv_get(resprop_frame, XV_KEY_DATA, VCS_RESPROP_KEY);
      p52 = xv_get(resprop_frame, XV_KEY_DATA, IVCS_RESPROP_KEY);
      allready_called = YES;
   }

   ext = xv_get(extprop_frame, XV_SHOW);
   prop = xv_get(prop_frame,XV_SHOW);
   res = xv_get(resprop_frame,XV_SHOW);

   /*check if prop_frame or extprop_frame sightable*/
   if((ext == TRUE) || (prop == TRUE) || (res == TRUE)) {

      /*get pd pointer*/
      if(mode == BY_SELECTED) {
	 if((pid = get_select()) < 0) {
	    pd = NULL;
	    return_value = -1;
	 }
	 else
	    if((pd = pdt_search(pid)) == NULL) {
	       clear_select();
	       return_value = -1;
	    }
      }

      if(prop == TRUE)
	 prop_items(pd);

      if(ext == TRUE)
	 ext_items(pd);

      if(res == TRUE)
	 res_items(pd);

      return(return_value);
   }
}


/* -------------------------------------------------------------------
 * function: prop_items()
 *           sets all panel items of the process properties frames
 * input:    pd  -> pointer to an pdt entry
 *                  if NULL deactivate all items
 * -------------------------------------------------------------------
 */
PRIVATE void prop_items(ProcessToDraw *pd)
{
   static int prop_deactivated = YES;
   static ProcessState psp;
   ProcessState *state;
   char timebuf[16];



   if((pd != NULL) || (prop_deactivated == NO)) {
      /*set or deactivate items*/
      if(pd == NULL) {
	 /* if pin out dismis frame*/
	 if(xv_get(prop_frame,FRAME_CMD_PIN_STATE) == FRAME_CMD_PIN_OUT)
	    xv_set(prop_frame,XV_SHOW,FALSE);
	 /* deactivate items*/
	 xv_set(p1, PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p2, PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p3, PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p4, PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p5, PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p6, PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p7, PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p8, PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p9, PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p10,PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 xv_set(p11,PANEL_VALUE,	"",
		PANEL_INACTIVE,	TRUE,	NULL);
	 prop_deactivated = YES;
      }
      else {
	 state = pd->pstate;
	 if(prop_deactivated == YES) {
	    xv_set(p1, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		itoa(state->pid),
		       NULL);
	    xv_set(p2, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		state->text,
		       NULL);
	    xv_set(p3, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		username(state->ruid),
		       NULL);
	    xv_set(p4, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		state->tty,
		       NULL);
	    xv_set(p5, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		state_2_str(state->status),
		       NULL);
	    xv_set(p6, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		itoa(state->tsize +
						     state->dsize +
						     state->ssize),
		       NULL);
	    xv_set(p7, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		itoa(state->rss),
		       NULL);
	    xv_set(p8, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		itoa(state->ucputime +
						     state->scputime),
		       NULL);
	    xv_set(p9, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		itoa(state->pctcpu),
		       NULL);
	    xv_set(p10, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		itoa(state->nice),
		       NULL);
	    strftime(timebuf, sizeof(timebuf), "%R %B %e", 
		     localtime(&state->start));
	    xv_set(p11, PANEL_INACTIVE,	FALSE,
		       PANEL_VALUE,		timebuf,
		       NULL);
	    prop_deactivated = NO;
	 }
	 else { /*write value only if something has been changed*/
	       if(state->pid != psp.pid) {
		  xv_set(p1, PANEL_VALUE,	itoa(state->pid), 	NULL);
		  xv_set(p3, PANEL_VALUE,	username(state->ruid), 	NULL);
	          strftime(timebuf, sizeof(timebuf), "%R %B %e", 
		           localtime(&state->start));
	          xv_set(p11,PANEL_VALUE,	timebuf, 		NULL);
	       }

	       if(strcmp(state->text, psp.text) != 0)
		  xv_set(p2, PANEL_VALUE,	state->text, 		NULL);

	       if(strcmp(state->tty, psp.tty) != 0)
		  xv_set(p4, PANEL_VALUE,	state->tty, 		NULL);

	       if(strcmp(state->status, psp.status) != 0)
		  xv_set(p5, PANEL_VALUE,	state_2_str(state->status),
		  NULL);

	       if((state->tsize != psp.tsize) || (state->dsize != psp.dsize) ||
		  (state->ssize != psp.ssize))
		  xv_set(p6, PANEL_VALUE,	itoa(state->tsize +
						     state->dsize +
						     state->ssize), 	NULL);

	       if(state->rss != psp.rss)
		  xv_set(p7, PANEL_VALUE,	itoa(state->rss), 	NULL);

	       if((state->ucputime != psp.ucputime) ||
		  (state->scputime != psp.scputime))
		  xv_set(p8, PANEL_VALUE,	itoa(state->ucputime +
						     state->scputime), 	NULL);

	       if(state->pctcpu != psp.pctcpu) {
		  sprintf(str,"%5.3f",state->pctcpu);
		  xv_set(p9, PANEL_VALUE,	str,		 	NULL);
	       }

	       if(state->nice != psp.nice)
		  xv_set(p10,PANEL_VALUE,	itoa(state->nice), 	NULL);

               /*store last showed ProcessState in psp*/
	       psp = *state;
	 }
      }
   }
}


/* -------------------------------------------------------------------
 * function: ext_items()
 *           sets all panel items of the extended process
 *	     properties frames
 * input:    pd  -> pointer to an pdt entry
 *                  if NULL deactivate all items
 * -------------------------------------------------------------------
 */
PRIVATE void ext_items(ProcessToDraw *pd)
{
   static int ext_deactivated = YES;
   static ProcessState esp;
   ProcessState *state;

   if((pd != NULL) || (ext_deactivated == NO)) {
	 /*set or deactivate items*/
	 if(pd == NULL) {
	    /* if pin out dismis frame*/
	    if(xv_get(extprop_frame,FRAME_CMD_PIN_STATE) == FRAME_CMD_PIN_OUT)
	       xv_set(extprop_frame,XV_SHOW,FALSE);
	    xv_set(p21, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
#ifndef linux
	    xv_set(p22, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p23, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
#endif
	    xv_set(p24, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p25, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p37, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p26, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p27, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p28, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p29, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p30,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p31,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p32,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p33,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p34,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p35,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p36,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    ext_deactivated = YES;
	 }
	 else {
	    state = pd->pstate;
	    if(ext_deactivated == YES) {
	       xv_set(p21,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->pri),
			  NULL);
#ifndef linux
	       xv_set(p22,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->restime),
			  NULL);
	       xv_set(p23,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->slptime),
			  NULL);
#endif
	       xv_set(p24,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		sigm_2_str(state->sigignore),
			  NULL);
	       xv_set(p25,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		sigm_2_str(state->sigcatch),
			  NULL);
	       xv_set(p37,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		sigm_2_str(state->sigblock),
			  NULL);
	       xv_set(p26,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->ppid),
			  NULL);
	       xv_set(p27,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->sid),
			  NULL);
	       xv_set(p28,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->pgrp),
			  NULL);
	       xv_set(p29,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->ruid),
			  NULL);
	       xv_set(p30,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->rgid),
			  NULL);
	       xv_set(p31,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->uid),
			  NULL);
	       xv_set(p32,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->gid),
			  NULL);
	       xv_set(p33,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->tsize),
			  NULL);
	       xv_set(p34,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->dsize),
			  NULL);
	       xv_set(p35,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->ssize),
			  NULL);
	       xv_set(p36,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		state->wchan,
			  NULL); 
	       ext_deactivated = NO;
	    }
	    else { /*write value only if something has been changed*/
	       if(state->pri != esp.pri)
		  xv_set(p21,PANEL_VALUE,	itoa(state->pri),	NULL);
#ifndef linux
	       if(state->restime != esp.restime)
		  xv_set(p22,PANEL_VALUE,	itoa(state->restime),	NULL);
	       if(state->slptime != esp.slptime)
		  xv_set(p23,PANEL_VALUE,	itoa(state->slptime),	NULL);
#endif
	       if(state->sigignore != esp.sigignore)
		  xv_set(p24,PANEL_VALUE,	sigm_2_str(state->sigignore),
		  NULL);
	       if(state->sigcatch != esp.sigcatch)
		  xv_set(p25,PANEL_VALUE,	sigm_2_str(state->sigcatch),
		  NULL);
	       if(state->sigcatch != esp.sigcatch)
		  xv_set(p37,PANEL_VALUE,	sigm_2_str(state->sigblock),
		  NULL);
	       if(state->ppid != esp.ppid)
		  xv_set(p26,PANEL_VALUE,	itoa(state->ppid),	NULL);
	       if(state->sid != esp.sid)
		  xv_set(p27,PANEL_VALUE,	itoa(state->sid),	NULL);
	       if(state->pgrp != esp.pgrp)
		  xv_set(p28,PANEL_VALUE,	itoa(state->pgrp),	NULL);
	       if(state->ruid != esp.ruid)
		  xv_set(p29,PANEL_VALUE,	itoa(state->ruid),	NULL);
	       if(state->rgid != esp.rgid)
		  xv_set(p30,PANEL_VALUE,	itoa(state->rgid),	NULL);
	       if(state->uid != esp.uid)
		  xv_set(p31,PANEL_VALUE,	itoa(state->uid),	NULL);
	       if(state->gid != esp.gid)
		  xv_set(p32,PANEL_VALUE,	itoa(state->gid),	NULL);
	       if(state->tsize != esp.tsize)
		  xv_set(p33,PANEL_VALUE,	itoa(state->tsize),	NULL);
	       if(state->dsize != esp.dsize)
		  xv_set(p34,PANEL_VALUE,	itoa(state->dsize),	NULL);
	       if(state->ssize != esp.ssize)
		  xv_set(p35,PANEL_VALUE,	itoa(state->ssize),	NULL);
	       if(state->wchan != esp.wchan)
		  xv_set(p36,PANEL_VALUE,	state->wchan,		NULL);

               /*store last showed ProcessState in esp*/
	       esp = *state;
	    }
	 }
   }
}


/* -------------------------------------------------------------------
 * function: res_items()
 *           sets all panel items of the resource usage process
 *	     properties frames
 * input:    pd  -> pointer to an pdt entry
 *                  if NULL deactivate all items
 * -------------------------------------------------------------------
 */
PRIVATE void res_items(ProcessToDraw *pd)
{
   static int res_deactivated = YES;
   static ProcessState rsp;
   ProcessState *state;

   if((pd != NULL) || (res_deactivated == NO)) {
	 /*set or deactivate items*/
	 if(pd == NULL) {
	    /* if pin out dismis frame*/
	    if(xv_get(resprop_frame,FRAME_CMD_PIN_STATE) == FRAME_CMD_PIN_OUT)
	       xv_set(resprop_frame,XV_SHOW,FALSE);
	    xv_set(p41, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p42, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p43, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p44, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p45, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p46, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p47, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p48, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p49, PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p50,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p51,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    xv_set(p52,PANEL_VALUE,	"",
		   PANEL_INACTIVE,	TRUE,	NULL);
	    res_deactivated = YES;
	 }
	 else {
	    state = pd->pstate;
	    if(res_deactivated == YES) {
	       xv_set(p41,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->ucputime),
			  NULL);
	       xv_set(p42,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->scputime),
			  NULL);
	       xv_set(p43,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->minflt),
			  NULL);
	       xv_set(p44,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->majflt),
			  NULL);
	       xv_set(p45,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->nswap),
			  NULL);
	       xv_set(p46,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->inblock),
			  NULL);
	       xv_set(p47,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->oublock),
			  NULL);
	       xv_set(p48,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->msgsnd),
			  NULL);
	       xv_set(p49,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->msgrcv),
			  NULL);
	       xv_set(p50,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->nsignals),
			  NULL);
	       xv_set(p51,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->nvcsw),
			  NULL);
	       xv_set(p52,PANEL_INACTIVE,	FALSE,
			  PANEL_VALUE,		itoa(state->nivcsw),
			  NULL);
	       res_deactivated = NO;
	    }
	    else { /*write value only if something has been changed*/
	       if(state->ucputime != rsp.ucputime)
		  xv_set(p41,PANEL_VALUE,	itoa(state->ucputime),	NULL);
	       if(state->scputime != rsp.scputime)
		  xv_set(p42,PANEL_VALUE,	itoa(state->scputime),	NULL);
	       if(state->minflt != rsp.minflt)
		  xv_set(p43,PANEL_VALUE,	itoa(state->minflt),	NULL);
	       if(state->majflt != rsp.majflt)
		  xv_set(p44,PANEL_VALUE,	itoa(state->majflt),	NULL);
	       if(state->nswap != rsp.nswap)
		  xv_set(p45,PANEL_VALUE,	itoa(state->nswap),	NULL);
	       if(state->inblock != rsp.inblock)
		  xv_set(p46,PANEL_VALUE,	itoa(state->inblock),	NULL);
	       if(state->oublock != rsp.oublock)
		  xv_set(p47,PANEL_VALUE,	itoa(state->oublock),	NULL);
	       if(state->msgsnd != rsp.msgsnd)
		  xv_set(p48,PANEL_VALUE,	itoa(state->msgsnd),	NULL);
	       if(state->msgrcv != rsp.msgrcv)
		  xv_set(p49,PANEL_VALUE,	itoa(state->msgrcv),	NULL);
	       if(state->nsignals != rsp.nsignals)
		  xv_set(p50,PANEL_VALUE,	itoa(state->nsignals),	NULL);
	       if(state->nvcsw != rsp.nvcsw)
		  xv_set(p51,PANEL_VALUE,	itoa(state->nvcsw),	NULL);
	       if(state->nivcsw != rsp.nivcsw)
		  xv_set(p52,PANEL_VALUE,	itoa(state->nivcsw),	NULL);

	       /*store last showed ProcessState in rsp*/
	       rsp = *state;
	    }
	 }
   }
}


/* -------------------------------------------------------------
 * function: sigm_2_str
 *           converts an mask integer like the ones in proc.h 
 *           named sigignore and sigcatch in a string containing 
 *           the appropriate signalnames
 * input:    sigmask
 * -------------------------------------------------------------
 */
PRIVATE char *sigm_2_str(int sigmask)
{
   char sig_string[300] = { 0 };
   int check = 1, i= 0;

   do {
      if((sigmask & check) != 0) {
	 strcat(sig_string, signals[i]);
	 strcat(sig_string, ", ");
      }
      i++;
   } while((check<<=1) != 0);

   return(sig_string);
}


/* -------------------------------------------------------------
 * function: state_2_str
 *		converts the short state string from the
 *		getpst module in a human readable string
 * input:    state->pointer to short state string
 * output:   pointer to readable string
 * -------------------------------------------------------------
 */
PRIVATE char *state_2_str(char *state)
{
   char *s_str[] = { "Sleeping",
		     "Running",
		     "Stoped",
		     "Idleing",
		     "not interuptable",
		     " and swaped" };
   char buf[50] = { 0 };

   switch(state[0]) {
      case 'S':
	 strcat(buf, "Sleeping");
	 break;
      case 'R':
	 strcat(buf, "Running");
	 break;
      case 'T':
	 strcat(buf, "Stoped");
	 break;
      case 'D':
	 strcat(buf, "Not interruptable");
	 break;
      case 'Z':
	 strcat(buf, "Zombie");
	 break;
      case 'I':
	 strcat(buf, "Idleing");
	 break;
   }
   switch(state[1]) {
      case 'W':
	 strcat(buf, " and swaped");
	 break;
   }

   return(buf);
}

