/**************************************************************************
 * Module name:   pst.h							  *
 * Project:       xps (Process State Programm for X			  *
 * written by:    Thomas Breitfeld (breitf@infotech.tu-chemnitz.de)	  *
 * last change:   11. 4. 1994                                             *
 *									  *
 * Copyright (c) 1994 Technical University of Chemnitz-Zwickau		  *
 *		      Department of Electrical Engineering		  *
 *		      Division of Data and Communication Technologies	  *
 *									  *
 * ---------------------------------------------------------------------- *
 * description:   this file contains the type definitions for the xps     *
 *		  process tables.				          *
 *		  and the function prototypes for the getpst module   	  *
 *		  							  *
 **************************************************************************/

#ifndef _PST_H
#define _PST_H

#ifdef linux
   #define MAXCOMLEN 32 /*max length of command name */
#else
   #include<sys/user.h>
#endif

#define STRING_SIZE 15 /*max number of characters in an alphanumarical
			  field*/

/*definition of the process status table (pst)*/
typedef struct _ProcessState {
/* name and id's */
	char text[MAXCOMLEN+1];    /*process command*/
	short pid; 		   /*process identification number*/
	short ppid;       	   /*parent pid*/
	short sid;		   /*session id*/
	short pgrp;		   /*name of process group leader*/
	short uid;        	   /*effective user id*/
	short gid;		   /*effective group id*/
	short ruid;        	   /*real user id*/
	short rgid;        	   /*real group id*/

/* scheduler information */
	int pri;	       	   /*priority*/
	int nice;		   /*nice value*/
	char restime;		   /*seconds resident*/
	char slptime;		   /*seconds sleeping*/

/* catched , blocked and ignored signals */
	int sigignore;		   /*signals being ignored*/
	int sigcatch;		   /*signals being catched*/
	int sigblock;		   /*signals beimg blocked*/

/* resource useage */
	int ucputime;		   /*seconds in user mode*/
	int scputime;		   /*seconds in system mode*/
	float pctcpu;		   /*%cpu using */
	int tsize;		   /*code size*/
	int dsize;		   /*data size*/
	int ssize;		   /*stack size*/
	int rss;   	   	   /*current resident size*/
	long minflt;		   /*any page faults not requiring I/O*/
	long majflt;		   /*any page faults requiring I/O*/
	long nswap;		   /*number of swaps*/
	long inblock;		   /*block input operations*/
	long oublock;		   /*block output operation*/
	long msgsnd;		   /*messages sent*/
	long msgrcv;		   /*messages received*/
	long nsignals;		   /*signals received*/
	long nvcsw;		   /*voluntary context switches*/
	long nivcsw;		   /*involuntary context switches*/

/* others */
	char status[3];		   /*state of process*/
	int flag;		   /*flag for spaecial information*/
	char wchan[16];   	   /*event on which process is waiting*/
	char tty[16];     	   /*process output device*/
	long start;		   /*start time*/
} ProcessState;


/*
 * function prototypes
 */
EXTERN int getpst_init(void);
EXTERN int getpst(ProcessState pst[]);	/*get process status table*/

#endif

