/**************************************************************************
 * Module name:   xpis.c						  *
 * Project:       xps (Process State Programm for X			  *
 * written by:    Thomas Breitfeld (breitf@infotech.tu-chemnitz.de)	  *
 * last change:   25. 8. 1994                                             *
 *									  *
 * Copyright (c) 1994 Technical University of Chemnitz-Zwickau		  *
 *		      Department of Electrical Engineering		  *
 *		      Division of Data and Communication Technologies	  *
 *									  *
 * ---------------------------------------------------------------------- *
 * description:   xpis.c contains the declarations of any globale	  *
 *		  variable and the main() function. The main function     *
 *		  initializes XView with all its objects and the timer .  *
 *		  Main starts the ball rolling by calling xv_main_loop()  *
 *		                                                   	  *
 * ---------------------------------------------------------------------- *
 **************************************************************************/

#include<stdio.h>
#include<sys/time.h>
#include<X11/Xlib.h>
#include<xview/xview.h>
#include<xview/notify.h>
#include<xview/frame.h>
#include<xview/panel.h>
#include<xview/canvas.h>
#include<xview/scrollbar.h>
#include<xview/xv_xrect.h>
#include<xview/cms.h>

#include"xpis.h"
#include"filter.h"
#include"xpsgc.h"
#include"paint.h"
#include"pst.h"
#include"keys.h"
#include"callback.h"
#include"control.h"
#include"icon.h"
#include"signames.h"

/* define control colors for Panel items */
#define CONTROL_RED 		0
#define NUM_MY_CONTROL_COLORS	1

PRIVATE Frame base_frame;


void main(int argc,char **argv)
{
   Frame	prop_frame, cust_frame, filter_frame, control_frame;
   Frame	extprop_frame, resprop_frame;
   Canvas	canvas;
   Panel 	panel;
   Panel_item	pitem1,pitem2,pitem3,pitem4,pitem5,pitem6,pitem7,pitem8;
   Panel_item   pitem9,pitem10,pitem11,pitem12,pitem13,pitem14,pitem15,pitem16;
   Panel_item   pitem17;
   Menu 	menu, sigmenu;
   Scrollbar 	h_scroll,v_scroll;
   Server_image replay_image, ff_image, fb_image, stop_image, goahead_image;
   bool		use_color = NO;
   XpisGC 	*xpisgc;
   char		label[20] = { "TB: " };
   char		hostname[15];
   int 		nproc;


   xv_init(XV_INIT_ARGC_PTR_ARGV,&argc,argv,NULL);

#ifdef linux
/*
 * process non-X-Options
 */
{ char *p;
   if(argc > 1) {
      for(p = argv[1]; *p; ++p) {
	 switch(*p) {
	    case '-': break;
	    case 'u':
	    case 'U': if(open_sys(argc > 2 ? argv[2] : NULL, YES) == -1) {
			 perror(argc > 2? argv[2] : "cannot open psdatabse");
			 exit(1);
		      }
		      exit(0);
	    default: fprintf(stderr, "Xps - unknown option: %s\n"
			     "usage: xps [-U [SYSTEM-PATH]] [X-Options]\n"
			     "-U --> update psdatabase\n"
			     "SYSTEM-PATH --> path of unstripped linux binary\n"
		     	     "X-Options --> all other X Options (see 'man X')\n",
			     argv[1]);
		     exit(1);
         }
      }
   }
}
#endif

/*
 * create base_frame
 */
   gethostname(hostname, sizeof(hostname));
   base_frame = (Frame)xv_create(XV_NULL,FRAME,
	FRAME_LABEL,		strcat(label,hostname),
	XV_WIDTH,		450,
	XV_HEIGHT,		300,
	FRAME_SHOW_FOOTER,	TRUE,
	WIN_CONSUME_EVENTS,     WIN_NO_EVENTS,	
				ACTION_OPEN, ACTION_CLOSE,
				NULL,
	WIN_EVENT_PROC,		base_frame_event_proc,
	NULL);


/*
 * create subframes
 */
   cust_frame = (Frame)xv_create(base_frame,FRAME_CMD,
	FRAME_LABEL,		"Customize Viewing",
	FRAME_SHOW_FOOTER,	TRUE,
	XV_KEY_DATA,		CUST_SHOWED_KEY, FALSE,
	NULL);

   prop_frame = (Frame)xv_create(base_frame,FRAME_CMD,
	FRAME_CMD_DEFAULT_PIN_STATE,FRAME_CMD_PIN_IN,
	FRAME_LABEL,		"Process Properties",
	FRAME_SHOW_FOOTER,	FALSE,
	XV_KEY_DATA,		PROP_SHOWED_KEY, FALSE,
	NULL);

   extprop_frame = (Frame)xv_create(base_frame,FRAME_CMD,
	FRAME_LABEL,		"Extended Informations",
	FRAME_SHOW_FOOTER,	FALSE,
	XV_KEY_DATA,		EXTPROP_SHOWED_KEY, FALSE,
	NULL);

   resprop_frame = (Frame)xv_create(base_frame,FRAME_CMD,
	FRAME_LABEL,		"Resource Usage",
	FRAME_SHOW_FOOTER,	FALSE,
	XV_KEY_DATA,		RESPROP_SHOWED_KEY, FALSE,
	NULL);

   filter_frame = (Frame)xv_create(base_frame,FRAME_CMD,
	FRAME_LABEL,		"Filter Adjustment",
	FRAME_SHOW_FOOTER,	TRUE,
	XV_KEY_DATA,		FILTER_SHOWED_KEY, FALSE,
	NULL);

   control_frame = (Frame)xv_create(base_frame,FRAME_CMD,
	FRAME_LABEL,		"XPS Control Centre",
	FRAME_CMD_DEFAULT_PIN_STATE,FRAME_CMD_PIN_IN,
	FRAME_SHOW_FOOTER,	FALSE,
	XV_KEY_DATA,		CONTROL_SHOWED_KEY, FALSE,
	NULL);

   xv_set(base_frame,
	XV_KEY_DATA,		PROP_FRAME_KEY, prop_frame,
	XV_KEY_DATA,		EXTPROP_FRAME_KEY, extprop_frame,
	XV_KEY_DATA,		CONTROL_FRAME_KEY, control_frame,
	XV_KEY_DATA,		RESPROP_FRAME_KEY, resprop_frame,
	NULL);


/*
 * create panel items for cust_frame
 */
   panel = (Panel)xv_get(cust_frame,FRAME_CMD_PANEL);
   xv_set(panel,PANEL_LAYOUT,PANEL_HORIZONTAL,NULL);
   #define CUST_ITEMS_X_POS 120

   pitem1 = (Panel_item)xv_create(panel,PANEL_CHOICE,
	PANEL_VALUE_X,		CUST_ITEMS_X_POS,
	PANEL_LABEL_STRING,	"Icons:",
	PANEL_CHOICE_STRINGS,	"Small","Regular","Fun",NULL,
	PANEL_VALUE,		1,
	NULL);

   pitem2 = (Panel_item)xv_create(panel,PANEL_CHOICE,
	PANEL_VALUE_X,		CUST_ITEMS_X_POS,
	PANEL_NEXT_ROW,		-1,
	PANEL_LABEL_STRING,	"Font Size:",
	PANEL_CHOICE_STRINGS,	"Small","Medium","Large","Extra Large",NULL,
	PANEL_VALUE,		1,
	NULL);

   pitem3 = (Panel_item)xv_create(panel,PANEL_CHOICE,
	PANEL_VALUE_X,		CUST_ITEMS_X_POS,
	PANEL_NEXT_ROW,		-1,
	PANEL_LABEL_STRING,	"Icon Animation:",
	PANEL_CHOICE_STRINGS,	"Yes","No",NULL,
	PANEL_VALUE,		0,
	NULL);

   xv_create(panel,PANEL_BUTTON,
        PANEL_NEXT_ROW,		25,
	PANEL_LABEL_X,		50,
	PANEL_LABEL_STRING,	"Apply",
	PANEL_NOTIFY_PROC,	button_cust_apply_proc,
	XV_KEY_DATA,		ICON_CUST_KEY, pitem1,
	XV_KEY_DATA,		FONT_CUST_KEY, pitem2,
	XV_KEY_DATA,		ANIMATE_CUST_KEY, pitem3,
	XV_KEY_DATA,		CUST_FRAME_KEY, cust_frame,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_ITEM_X_GAP,	25,
	PANEL_LABEL_STRING,	"Default",
	PANEL_NOTIFY_PROC,	button_cust_def_proc,
	XV_KEY_DATA,		ICON_CUST_KEY, pitem1,
	XV_KEY_DATA,		FONT_CUST_KEY, pitem2,
	XV_KEY_DATA,		ANIMATE_CUST_KEY, pitem3,
	NULL);

   window_fit(panel);
   window_fit(cust_frame);


/*
 * create panel items for filter_frame
 */
   panel = (Panel)xv_get(filter_frame,FRAME_CMD_PANEL);
   xv_set(panel,PANEL_LAYOUT,PANEL_HORIZONTAL,NULL);

   pitem1 = (Panel_item)xv_create(panel,PANEL_CHOICE,
	PANEL_LABEL_STRING,	"Users:",
	PANEL_LABEL_WIDTH,	45,
	PANEL_CHOICE_STRINGS,	"All","Only",NULL,
	PANEL_VALUE,		U_ONLY,
	PANEL_NOTIFY_PROC,	panel_filter_user_proc,
	NULL);

   pitem2 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		170,
	PANEL_LABEL_STRING,	":",
	PANEL_VALUE_DISPLAY_WIDTH,190,
	PANEL_VALUE_STORED_LENGTH,FILTER_STORED_LENGTH,
	PANEL_VALUE,		username(getuid()),
	NULL);

   pitem3 = (Panel_item)xv_create(panel,PANEL_CHOICE,
	PANEL_NEXT_ROW,		-1,
	PANEL_LABEL_STRING,	"TTY's:",
	PANEL_LABEL_WIDTH,	45,
	PANEL_CHOICE_STRINGS,	"All","Without",NULL,
	PANEL_VALUE,		TTY_WITHOUT,
	PANEL_NOTIFY_PROC,	panel_filter_tty_proc,
	NULL);

   pitem4 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		170,
	PANEL_LABEL_STRING,	":",
	PANEL_VALUE_DISPLAY_WIDTH,190,
	PANEL_VALUE_STORED_LENGTH,FILTER_STORED_LENGTH,
	PANEL_VALUE,		"?",
	NULL);

   pitem5 = (Panel_item)xv_create(panel,PANEL_CHOICE,
	PANEL_NEXT_ROW,		-1,
	PANEL_LABEL_STRING,	"States:",
	PANEL_LABEL_WIDTH,	45,
	PANEL_CHOICE_STRINGS,	"All","Only",NULL,
	PANEL_DEFAULT_VALUE,	STATE_ALL,
	PANEL_NOTIFY_PROC,	panel_filter_state_proc,
	NULL);

   pitem6 = (Panel_item)xv_create(panel,PANEL_TOGGLE,
	PANEL_CHOICE_NROWS,	3,
	PANEL_VALUE_X,		170,
	PANEL_LABEL_STRING,	":",
	PANEL_CHOICE_STRINGS,	"Runing","Sleeping","Idleing",
				"Swaped out","Stopped","Waiting",NULL,
        PANEL_INACTIVE,		TRUE,
	NULL);

   xv_set(pitem1,
	XV_KEY_DATA,		USER_TEXT_FILTER_KEY,	pitem2,
	NULL);

   xv_set(pitem3,
	XV_KEY_DATA,		TTY_TEXT_FILTER_KEY,	pitem4,
	NULL);

   xv_set(pitem5,
	XV_KEY_DATA,		STATE_TOGGLE_FILTER_KEY,pitem6,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_NEXT_ROW,		20,
	PANEL_LABEL_X,		60,
	PANEL_LABEL_STRING,	"Apply",
	PANEL_NOTIFY_PROC,	button_filter_apply_proc,
	XV_KEY_DATA,		USER_FILTER_KEY,      pitem1,
	XV_KEY_DATA,		USER_TEXT_FILTER_KEY, pitem2,
	XV_KEY_DATA,		TTY_FILTER_KEY,       pitem3,
	XV_KEY_DATA,		TTY_TEXT_FILTER_KEY,  pitem4,
	XV_KEY_DATA,		STATE_FILTER_KEY,     pitem5,
	XV_KEY_DATA,		STATE_TOGGLE_FILTER_KEY,pitem6,
	XV_KEY_DATA,		FILTER_FRAME_KEY,    filter_frame,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_ITEM_X_GAP,	20,
	PANEL_LABEL_STRING,	"Default",
	PANEL_NOTIFY_PROC,	button_filter_def_proc,
	XV_KEY_DATA,		USER_FILTER_KEY,      pitem1,
	XV_KEY_DATA,		USER_TEXT_FILTER_KEY, pitem2,
	XV_KEY_DATA,		TTY_FILTER_KEY,       pitem3,
	XV_KEY_DATA,		TTY_TEXT_FILTER_KEY,  pitem4,
	XV_KEY_DATA,		STATE_FILTER_KEY,     pitem5,
	XV_KEY_DATA,		STATE_TOGGLE_FILTER_KEY,pitem6,
	NULL);

   window_fit(panel);
   window_fit(filter_frame);


/*
 * create panel items for prop_frame
 */
   #define PROP_PANELS_X_POS 150
   #define PROP_TEXT_WIDTH   150
   panel = (Panel)xv_get(prop_frame,FRAME_CMD_PANEL);
   xv_set(panel,PANEL_LAYOUT,PANEL_VERTICAL,NULL);

   pitem1 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Process ID:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem2 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Command:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem3 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"User Name:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem4 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Attached to TTY:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem5 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"State:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem6 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Over All Size:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem7 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"resident Size:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem8 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"used CPU Time",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem9 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"CPU usage in %",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem10 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Nice Value:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem11 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Start time:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_NEXT_ROW,		20,
	PANEL_LABEL_X,		60,
	PANEL_LABEL_STRING,	"Extended Informations",
	PANEL_NOTIFY_PROC,	button_extended_proc,
	XV_KEY_DATA,		EXTPROP_FRAME_KEY, extprop_frame,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_NEXT_ROW,		-1,
	PANEL_LABEL_X,		60,
	PANEL_LABEL_STRING,	"Resource Usage",
	PANEL_NOTIFY_PROC,	button_resource_proc,
	XV_KEY_DATA,		RESPROP_FRAME_KEY, resprop_frame,
	NULL);

   xv_set(prop_frame,
	XV_KEY_DATA,		PID_PROP_KEY,	pitem1,
	XV_KEY_DATA,		COM_PROP_KEY,	pitem2,
	XV_KEY_DATA,		UNAME_PROP_KEY, pitem3,
	XV_KEY_DATA,		TTY_PROP_KEY,	pitem4,
	XV_KEY_DATA,		STATE_PROP_KEY, pitem5,
	XV_KEY_DATA,		SIZE_PROP_KEY,	pitem6,
	XV_KEY_DATA,		RSS_PROP_KEY,	pitem7,
	XV_KEY_DATA,		TIME_PROP_KEY,	pitem8,
	XV_KEY_DATA,		PCT_PROP_KEY,	pitem9,
	XV_KEY_DATA,		NICE_PROP_KEY,	pitem10,
	XV_KEY_DATA,		START_PROP_KEY,	pitem11,
	NULL);

   window_fit(panel);
   window_fit(prop_frame);


/*
 * create panel items for extprop_frame
 */
   #define PROP_PANELS_X_POS 150
   #define PROP_TEXT_WIDTH   150
   panel = (Panel)xv_get(extprop_frame,FRAME_CMD_PANEL);
   xv_set(panel,PANEL_LAYOUT,PANEL_VERTICAL,NULL);

   pitem1 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Priority:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);
#ifndef linux
   pitem2 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Seconds resident:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem3 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Seconds sleeping:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);
#endif

   pitem4 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Parent Process ID:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem5 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Session ID:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem6 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Process Group ID:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem7 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Real User ID:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem8 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Real Group ID:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem9 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Effective User ID:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem10 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Effective Group ID:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem11 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Signals to ignore:",
	PANEL_VALUE_STORED_LENGTH,300,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem12 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Signals to catch:",
	PANEL_VALUE_STORED_LENGTH,300,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem17 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Signals to block:",
	PANEL_VALUE_STORED_LENGTH,300,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem13 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Code Size:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem14 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Data Size:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem15 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Stack Size:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem16 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		PROP_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Wait Channel:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,PROP_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   xv_set(extprop_frame,
	XV_KEY_DATA,		PRI_EXTPROP_KEY,	pitem1,
#ifndef linux
	XV_KEY_DATA,		RESTIME_EXTPROP_KEY,	pitem2,
	XV_KEY_DATA,		SLPTIME_EXTPROP_KEY, 	pitem3,
#endif
	XV_KEY_DATA,		PPID_EXTPROP_KEY,	pitem4,
	XV_KEY_DATA,		SID_EXTPROP_KEY,	pitem5,
	XV_KEY_DATA,		PGRP_EXTPROP_KEY,	pitem6,
	XV_KEY_DATA,		RUID_EXTPROP_KEY,	pitem7,
	XV_KEY_DATA,		RGID_EXTPROP_KEY,	pitem8,
	XV_KEY_DATA,		EUID_EXTPROP_KEY,	pitem9,
	XV_KEY_DATA,		EGID_EXTPROP_KEY,	pitem10,
	XV_KEY_DATA,		SIGIGNORE_EXTPROP_KEY,	pitem11,
	XV_KEY_DATA,		SIGCATCH_EXTPROP_KEY,	pitem12,
        XV_KEY_DATA,		SIGBLOCK_EXTPROP_KEY,   pitem17,
	XV_KEY_DATA,		CSIZE_EXTPROP_KEY,	pitem13,
	XV_KEY_DATA,		DSIZE_EXTPROP_KEY,	pitem14,
	XV_KEY_DATA,		SSIZE_EXTPROP_KEY,	pitem15,
	XV_KEY_DATA,		WCH_EXTPROP_KEY,	pitem16,
	NULL);

   window_fit(panel);
   window_fit(extprop_frame);


/*
 * create panel items for resprop_frame
 */
   #define RES_PANELS_X_POS 215
   #define RES_TEXT_WIDTH   100
   panel = (Panel)xv_get(resprop_frame,FRAME_CMD_PANEL);
   xv_set(panel,PANEL_LAYOUT,PANEL_VERTICAL,NULL);

   pitem1 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Seconds user mode:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem2 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Seconds sys mode:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem3 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Page faults not req. I/O:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem4 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Page faults req. I/O:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem5 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Swaps:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem6 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Block input operations:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem7 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Block output operations:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem8 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Massages sent:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem9 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Massages received:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem10 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Signals received:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem11 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Voluntary context switches:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   pitem12 = (Panel_item)xv_create(panel,PANEL_TEXT,
	PANEL_VALUE_X,		RES_PANELS_X_POS,
	PANEL_LABEL_STRING,	"Involuntary context switches:",
	PANEL_VALUE_STORED_LENGTH,PROP_STORED_LENGTH,
	PANEL_VALUE_DISPLAY_WIDTH,RES_TEXT_WIDTH,
	PANEL_INACTIVE,		TRUE,
	NULL);

   xv_set(resprop_frame,
	XV_KEY_DATA,		UCPUTIME_RESPROP_KEY,	pitem1,
	XV_KEY_DATA,		SCPUTIME_RESPROP_KEY,	pitem2,
	XV_KEY_DATA,		PFIO_RESPROP_KEY, 	pitem3,
	XV_KEY_DATA,		PFNIO_RESPROP_KEY,	pitem4,
	XV_KEY_DATA,		SWAPS_RESPROP_KEY,	pitem5,
	XV_KEY_DATA,		BLOCKIN_RESPROP_KEY,	pitem6,
	XV_KEY_DATA,		BLOCKOUT_RESPROP_KEY,	pitem7,
	XV_KEY_DATA,		MSENT_RESPROP_KEY,	pitem8,
	XV_KEY_DATA,		MRECV_RESPROP_KEY,	pitem9,
	XV_KEY_DATA,		SIGNALS_RESPROP_KEY,	pitem10,
	XV_KEY_DATA,		VCS_RESPROP_KEY,	pitem11,
	XV_KEY_DATA,		IVCS_RESPROP_KEY,	pitem12,
	NULL);

   window_fit(panel);
   window_fit(resprop_frame);


/*
 * create serverimages for button labeling in control frame
 */
   {
      unsigned short replay_image_bits[] = { 
#include"icons/replay.icon" 
      };
      unsigned short ff_image_bits[] = { 
#include"icons/ff.icon" 
      };
      unsigned short fb_image_bits[] = { 
#include"icons/fb.icon" 
      };
      unsigned short stop_image_bits[] = { 
#include"icons/stop.icon" 
      };
      unsigned short goahead_image_bits[] = { 
#include"icons/goahead.icon" 
      };

      replay_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
	XV_WIDTH,		32,
	XV_HEIGHT,		32,
	SERVER_IMAGE_BITS,	replay_image_bits,
	NULL);

      ff_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
	XV_WIDTH,		32,
	XV_HEIGHT,		32,
	SERVER_IMAGE_BITS,	ff_image_bits,
	NULL);

      fb_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
	XV_WIDTH,		32,
	XV_HEIGHT,		32,
	SERVER_IMAGE_BITS,	fb_image_bits,
	NULL);

      stop_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
	XV_WIDTH,		32,
	XV_HEIGHT,		32,
	SERVER_IMAGE_BITS,	stop_image_bits,
	NULL);

      goahead_image = (Server_image)xv_create(XV_NULL, SERVER_IMAGE,
	XV_WIDTH,		32,
	XV_HEIGHT,		32,
	SERVER_IMAGE_BITS,	goahead_image_bits,
	NULL);
   }


/*
 * create panel items for control_frame
 */
   panel = (Panel)xv_get(control_frame,FRAME_CMD_PANEL);
   xv_set(panel,PANEL_LAYOUT,PANEL_HORIZONTAL,NULL);
   #define CONTROL_PANELS_X_POS 220
   #define CONTROL_PANELS_DISP_WIDTH 4

   pitem1 = (Panel_item)xv_create(panel,PANEL_NUMERIC_TEXT,
	PANEL_VALUE_X,		CONTROL_PANELS_X_POS,
	PANEL_VALUE_DISPLAY_LENGTH,CONTROL_PANELS_DISP_WIDTH,
	PANEL_LABEL_STRING,	"Xps Timer:            Seconds:",
	PANEL_VALUE,		1,
	PANEL_MIN_VALUE,	0,
	PANEL_MAX_VALUE,	10,
	NULL);

   pitem2 = (Panel_item)xv_create(panel,PANEL_NUMERIC_TEXT,
	PANEL_NEXT_ROW,		3,
	PANEL_VALUE_X,		CONTROL_PANELS_X_POS,
	PANEL_VALUE_DISPLAY_LENGTH,CONTROL_PANELS_DISP_WIDTH,
	PANEL_LABEL_STRING,	"1/10 Seconds:",
	PANEL_VALUE,		0,
	PANEL_MIN_VALUE,	0,
	PANEL_MAX_VALUE,	9,
	NULL);

   pitem3 = (Panel_item)xv_create(panel,PANEL_NUMERIC_TEXT,
	PANEL_NEXT_ROW,		-1,
	PANEL_VALUE_X,		CONTROL_PANELS_X_POS,
	PANEL_VALUE_DISPLAY_LENGTH,CONTROL_PANELS_DISP_WIDTH,
	PANEL_LABEL_STRING,	"Replay Timer:            Seconds:",
	PANEL_VALUE,		2,
	PANEL_MIN_VALUE,	1,
	PANEL_MAX_VALUE,	10,
	NULL);

   pitem4 = (Panel_item)xv_create(panel,PANEL_NUMERIC_TEXT,
	PANEL_NEXT_ROW,		-1,
	PANEL_ITEM_Y_GAP,	10,
	PANEL_VALUE_X,		CONTROL_PANELS_X_POS,
	PANEL_VALUE_DISPLAY_LENGTH,CONTROL_PANELS_DISP_WIDTH,
	PANEL_LABEL_STRING,	"Number of Xps Images to hold:",
	PANEL_VALUE,		20,
	PANEL_MIN_VALUE,	2,
	PANEL_MAX_VALUE,	100,
	NULL);

   pitem5 = (Panel_item)xv_create(panel,PANEL_SLIDER,
	PANEL_NEXT_ROW,		25,
	PANEL_LABEL_STRING,	"Image:",
	PANEL_VALUE_DISPLAY_LENGTH,CONTROL_PANELS_DISP_WIDTH,
	PANEL_VALUE,		0,
	PANEL_MIN_VALUE,	-9,
	PANEL_MAX_VALUE,	0,
	PANEL_SLIDER_WIDTH,	150,
	PANEL_TICKS,		5,
	PANEL_NOTIFY_PROC,	slider_control_proc,
	PANEL_NOTIFY_LEVEL,	PANEL_ALL,
	PANEL_INACTIVE,		YES,
	NULL);

   pitem9 = (Panel_item)xv_create(panel,PANEL_BUTTON,
	PANEL_NEXT_ROW,		15,
	PANEL_LABEL_X,		30,
	PANEL_LABEL_IMAGE,      goahead_image,
	PANEL_NOTIFY_PROC,	goahead_button_control_proc,
	NULL);

   pitem6 = (Panel_item)xv_create(panel,PANEL_BUTTON,
	PANEL_LABEL_X,		105,
	PANEL_LABEL_IMAGE,	fb_image,
	PANEL_NOTIFY_PROC,	fb_button_control_proc,
	PANEL_INACTIVE,		YES,
	NULL);

   pitem7 = (Panel_item)xv_create(panel,PANEL_BUTTON,
	PANEL_LABEL_IMAGE,	stop_image,
	PANEL_LABEL_WIDTH,	60,
	PANEL_NOTIFY_PROC,	stop_button_control_proc,
	NULL);

   pitem8 = (Panel_item)xv_create(panel,PANEL_BUTTON,
	PANEL_LABEL_IMAGE,	ff_image,
	PANEL_NOTIFY_PROC,	ff_button_control_proc,
	PANEL_INACTIVE,		YES,
	NULL);

   xv_set(pitem7,
	XV_KEY_DATA,		STOP_IMAGE_KEY,	     stop_image,
	XV_KEY_DATA,		REPLAY_IMAGE_KEY,    replay_image,
	XV_KEY_DATA,		REPLAY_TIMER_KEY,    pitem3,
	XV_KEY_DATA,		SLIDER_CONTROL_KEY,  pitem5,
	XV_KEY_DATA,		FB_CONTROL_KEY,	     pitem6,
	XV_KEY_DATA,		FF_CONTROL_KEY,	     pitem8,
	NULL);

   xv_set(pitem9,
	XV_KEY_DATA,		STOP_IMAGE_KEY,	     stop_image,
	XV_KEY_DATA,		XPS_TIMER_SEC_KEY,   pitem1,
	XV_KEY_DATA,		XPS_TIMER_10SEC_KEY, pitem2,
	XV_KEY_DATA,		NR_IMAGES_KEY,	     pitem4,
	XV_KEY_DATA,		SLIDER_CONTROL_KEY,  pitem5,
	XV_KEY_DATA,		FB_CONTROL_KEY,	     pitem6,
	XV_KEY_DATA,		REPLAY_CONTROL_KEY,  pitem7,
	XV_KEY_DATA,		FF_CONTROL_KEY,	     pitem8,
	NULL);

   xv_set(control_frame,
	XV_KEY_DATA,		SLIDER_CONTROL_KEY,  pitem5,
	XV_KEY_DATA,		FB_CONTROL_KEY,	     pitem6,
	XV_KEY_DATA,		REPLAY_CONTROL_KEY,  pitem7,
	XV_KEY_DATA,		FF_CONTROL_KEY,	     pitem8,
	NULL);

   window_fit(panel);
   window_fit(control_frame);


/*
 * create base_frame panel and canvas
 */
   panel = (Panel)xv_create(base_frame,PANEL,
	PANEL_LAYOUT,	PANEL_HORIZONTAL,
	NULL) ;

   menu = (Menu)xv_create(XV_NULL,MENU,
	MENU_GEN_PIN_WINDOW,	base_frame,	"Views",
	MENU_ITEM,
	      MENU_STRING,	"Tree",
	      MENU_NOTIFY_PROC,	menu_tree_proc,
	      NULL,
	MENU_ITEM,
	      MENU_STRING,	"List",
	      MENU_NOTIFY_PROC,	menu_list_proc,
	      NULL,
	MENU_ITEM,
	      MENU_STRING,	"",
	      MENU_INACTIVE,	TRUE,
	      NULL,
	MENU_ITEM,
              MENU_STRING,	"show childs",
	      MENU_NOTIFY_PROC,	menu_child_proc,
	      NULL,
	MENU_ITEM,
	      MENU_STRING,	"show all",
	      MENU_NOTIFY_PROC,	menu_all_proc,
	      NULL,
	MENU_ITEM,
	      MENU_STRING,	"",
	      MENU_INACTIVE,	TRUE,
	      NULL,
	MENU_ITEM,
	      MENU_STRING,	"customize...",
	      MENU_NOTIFY_PROC,	menu_customize_proc,
	      XV_KEY_DATA,	CUST_FRAME_KEY,	cust_frame,
	      XV_KEY_DATA,	BASE_FRAME_KEY,	base_frame,
	      NULL,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,	"View",
	PANEL_ITEM_MENU,	menu,
	NULL);

   sigmenu = (Menu)xv_create(XV_NULL,MENU,
	MENU_GEN_PIN_WINDOW,	base_frame, "Signals",
	MENU_NOTIFY_PROC,	menu_signal_proc,
	NULL);
   {
      char **s;
      Menu_item mi;

      for(s=signals; **s!='\0'; s++) {
	 mi = (Menu_item)xv_create(XV_NULL, MENUITEM,
		MENU_STRING,	*s,
		MENU_RELEASE,
		NULL);
	 xv_set(sigmenu, MENU_APPEND_ITEM, mi, NULL);
      }
   }

   menu = (Menu)xv_create(XV_NULL,MENU,
	MENU_GEN_PIN_WINDOW,	base_frame,	"Actions",
	MENU_NOTIFY_PROC,	menu_action_proc,
	MENU_STRINGS,		"Soft Kill","Hard Kill",NULL,
	MENU_PULLRIGHT_ITEM, 	"Signal to Process",  sigmenu,
	MENU_PULLRIGHT_ITEM,	"Signal to Group", sigmenu,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,	"Action",
	PANEL_ITEM_MENU,	menu,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,	"Control...",
	PANEL_NOTIFY_PROC,	button_control_proc,
	XV_KEY_DATA,		CONTROL_FRAME_KEY, control_frame,
	XV_KEY_DATA,		BASE_FRAME_KEY, base_frame,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,	"Properties...",
	PANEL_NOTIFY_PROC,	button_prop_proc,
	XV_KEY_DATA,		PROP_FRAME_KEY, prop_frame,
	XV_KEY_DATA,		BASE_FRAME_KEY, base_frame,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,	"Filter...",
	PANEL_NOTIFY_PROC,	button_filter_proc,
	XV_KEY_DATA,		FILTER_FRAME_KEY, filter_frame,
	XV_KEY_DATA,		BASE_FRAME_KEY, base_frame,
	NULL);

   xv_create(panel,PANEL_BUTTON,
	PANEL_LABEL_STRING,	"Info",
	PANEL_NOTIFY_PROC,	button_info,
	XV_KEY_DATA,		BASE_FRAME_KEY,	base_frame,
	NULL);

   window_fit_height(panel);

   canvas = (Canvas)xv_create(base_frame, CANVAS,
	OPENWIN_AUTO_CLEAR,	FALSE,
	CANVAS_REPAINT_PROC,	repaint,
	CANVAS_X_PAINT_WINDOW,  FALSE,
	CANVAS_AUTO_EXPAND,	FALSE,
	CANVAS_AUTO_SHRINK,	FALSE,
	CANVAS_RETAINED,	TRUE,
	NULL);

   h_scroll = (Scrollbar)xv_create(canvas,SCROLLBAR,
	SCROLLBAR_DIRECTION,	SCROLLBAR_HORIZONTAL,
	SCROLLBAR_PIXELS_PER_UNIT,20,
	SCROLLBAR_SPLITTABLE,	TRUE,
	NULL);

   v_scroll = (Scrollbar)xv_create(canvas,SCROLLBAR,
	SCROLLBAR_DIRECTION,	SCROLLBAR_VERTICAL,
	SCROLLBAR_PIXELS_PER_UNIT,20,
	SCROLLBAR_SPLITTABLE,	TRUE,
	NULL);

   window_fit(base_frame);


/*
 * init gc for painting
 */
   if((xpisgc = initgc(canvas,I_REGULAR,F_MEDIUM,YES)) == NULL) {
      puterr("Xps: unable to create gc, programm terminated\n");
      exit(1);
   }


/* 
 * set canvas Input Mask and color map segment
 */
   xv_set(canvas_paint_window(canvas),
	WIN_CMS,		xpisgc->cms,
	WIN_EVENT_PROC,		canvas_event_proc,
	WIN_CONSUME_EVENTS,
		WIN_NO_EVENTS,WIN_ASCII_EVENTS,
		WIN_MOUSE_BUTTONS,KBD_DONE,KBD_USE,
		NULL,
	NULL);


/*
 * init base frame icons
 */
   init_icons(base_frame,hostname);


/*
 * init getpst module
 */
      if((nproc = getpst_init()) <= 0) {
	 fprintf(stderr,"Xps: getpst_init failed with nproc=%d !\n"
			"Programm terminated!\n", nproc);
	 exit(1);
      }


/*
 * init recorder track
 */
   if(control_init(10, nproc) < 0) {
      puterr("Xps: no memory for recorder track! Programm terminated!\n");
      exit(1);
   }


/*
 * init filter module
 */
   {
      int uids[] = { 0 ,END_F_UIDS_LIST };
      char *ttys[] = { "? ",NULL };
      States states = { "",
			"" };
      uids[0] = getuid();

      if(filter_init(nproc, uids, ttys, &states) < 0) {
	 puterr("Xps: filter_init failed -> no memory!\n"
		"Programm terminated\n");
	 exit(1);
      }
   }


/*
 * init paint module
 */
   if(paint_init(nproc) < 0) {
      puterr("Xps: paint_init failed because of no memory!\n"
	     "Programm terminated\n");
      exit(1);
   }


   xv_main_loop(base_frame);
}


Frame get_base_frame(void)
{
   return(base_frame);
}
