/**************************************************************************
 * Module name:   xpsgc.c			                          *
 * Project:       xps (Process State Programm for X			  *
 * written by:    Thomas Breitfeld (breitf@infotech.tu-chemnitz.de)	  *
 * last change:   11. 4. 1994                                             *
 *									  *
 * Copyright (c) 1994 Technical University of Chemnitz-Zwickau		  *
 *		      Department of Electrical Engineering		  *
 *		      Division of Data and Communication Technologies	  *
 *									  *
 * ---------------------------------------------------------------------- *
 * description:   This function contains the definition of the Xps	  *
 *		  Graphic Context and the function to create, to change   *
 *		  and to get it.					  *
 *                					                  *
 * ---------------------------------------------------------------------- *
 * entry point:   initgc()						  *
 **************************************************************************/

#include"xpis.h"
#include"xpsgc.h"

#include"icons/Z_small.icon"
#include"icons/R_small.icon"
#include"icons/S_small.icon"
#include"icons/T_small.icon"
#include"icons/W_small.icon"
#include"icons/K_small.icon"
#include"icons/Z_medium.icon"
#include"icons/R_medium.icon"
#include"icons/S_medium.icon"
#include"icons/T_medium.icon"
#include"icons/W_medium.icon"
#include"icons/K_medium.icon"
#include"icons/Z_fun.icon"
#include"icons/R_fun.icon"
#include"icons/S_fun.icon"
#include"icons/T_fun.icon"
#include"icons/W_fun.icon"
#include"icons/K_fun.icon"


/* -----------------------------------------------------------------
 * function: initgc
 *           initializes the X graphics context to use with Xlib
 *           graphic functions, including the fonts
 *	     and all needed Pixmaps
 * input:    *dpy pointer to display
 * return:   pointer to XpisGC
 *	     or NULL if init failed
 * -----------------------------------------------------------------
 */
PUBLIC XpisGC *initgc(Canvas canvas,int iconstyle,int fontsize,int tree)
{
   static XpisGC xpisgc;
   static Images small_images, reg_images, fun_images;
   static bool small_init = NO, reg_init = NO, fun_init = NO;
   static bool allready_called = NO;
   static Xv_singlecolor colors[] = {
				       { 255, 255, 255 }, /* white */
				       { 234, 124, 124 }, /* red */
				       { 139, 192, 73  }, /* green */
				       { 115, 189, 230 }, /* blue */
				       { 255, 220, 138 }, /* yellow */
				       { 228, 141, 231 }, /* purple */
				       { 80,  224, 179 }, /* cyan */
				       {   0,   0,   0 }  /* black */
				    };
   XGCValues gcvalues;
   Xv_font font;

   /* get X structures to use X graphic routines,
    * create color map segment
    * and set initial graphic context
    */
   if(allready_called == NO) {
      if(canvas != 0) {
	 xpisgc.canvas = canvas;
	 xpisgc.dpy = (Display *)xv_get(canvas,XV_DISPLAY);
	 xpisgc.xwin = (Window)xv_get(canvas_paint_window(canvas),XV_XID);

      /* check if color graphic system and set use_color appropriatly
       */
	 if(DefaultDepth(xpisgc.dpy, DefaultScreen(xpisgc.dpy)) > 1)
	    xpisgc.use_color = YES;
	 else
	    xpisgc.use_color = NO;

	 xpisgc.cms = (Cms)xv_create(XV_NULL, CMS,
		CMS_SIZE,	NUM_COLORS,
		CMS_COLORS,	colors,
		NULL);

      /* get the actual indices into the colormap
       */
	 xpisgc.color_table = (unsigned long *)xv_get(xpisgc.cms,
						      CMS_INDEX_TABLE);

      /* Create a GC to use with Xlib graphics -- set the fg/bg colors
       * and set the Font, which is the XV_XID of the XView font object.
       */
	 gcvalues.foreground = xpisgc.color_table[BLACK];
	 gcvalues.background = xpisgc.color_table[WHITE];
	 gcvalues.graphics_exposures = False;
	 xpisgc.gc = XCreateGC(xpisgc.dpy, 
			RootWindow(xpisgc.dpy,DefaultScreen(xpisgc.dpy)),
			GCForeground | GCBackground | GCGraphicsExposures,
			&gcvalues);

      /* Create another gc with inverted colors 
       */
	 gcvalues.foreground = xpisgc.color_table[WHITE];
	 gcvalues.background = xpisgc.color_table[BLACK];
	 xpisgc.invgc = XCreateGC(xpisgc.dpy,
			   RootWindow(xpisgc.dpy,DefaultScreen(xpisgc.dpy)),
			   GCForeground | GCBackground | GCGraphicsExposures,
			   &gcvalues);

	 allready_called = YES;
      }
      else {
	 puterr("initgc: can't create gc because canvas = 0 !\n");
	 return(NULL);
      }
   }


   /* Set font to use with Xlib graphics
    */
   if(fontsize != CHANGE_NOT) {
      switch(fontsize) {
	 case F_SMALL:
	    font = (Xv_Font)xv_find(canvas, FONT, 
			FONT_FAMILY,	FONT_FAMILY_DEFAULT,
			FONT_STYLE,	FONT_STYLE_DEFAULT,
			FONT_SIZES_FOR_SCALE,8,10,12,19,
			FONT_SCALE,	WIN_SCALE_SMALL,
			NULL);
	    break;
	 case F_MEDIUM:
	    font = (Xv_Font)xv_find(canvas, FONT, 
			FONT_FAMILY,	FONT_FAMILY_DEFAULT,
			FONT_STYLE,	FONT_STYLE_DEFAULT,
			FONT_SIZES_FOR_SCALE,8,10,12,19,
			FONT_SCALE,	WIN_SCALE_MEDIUM,
			NULL);
	    break;
	 case F_LARGE:
	    font = (Xv_Font)xv_find(canvas, FONT, 
			FONT_FAMILY,	FONT_FAMILY_DEFAULT,
			FONT_STYLE,	FONT_STYLE_DEFAULT,
			FONT_SIZES_FOR_SCALE,8,10,12,19,
			FONT_SCALE,	WIN_SCALE_LARGE,
			NULL);
	    break;
	 case F_EXTRALARGE:
	    font = (Xv_Font)xv_find(canvas, FONT, 
			FONT_FAMILY,	FONT_FAMILY_DEFAULT,
			FONT_STYLE,	FONT_STYLE_DEFAULT,
			FONT_SIZES_FOR_SCALE,8,10,12,19,
			FONT_SCALE,	WIN_SCALE_EXTRALARGE,
			NULL);
	    break;
	 default:
	    puterr("initgc: unknown font size specified in funktion call\n");
      }
      if((xpisgc.xfontstruct=(XFontStruct *)xv_get(font,FONT_INFO)) == NULL) {
	 puterr("initgc: xfontstructpointer = null\n");
	 return(NULL);
      }
      XSetFont(xpisgc.dpy,xpisgc.gc,xpisgc.xfontstruct->fid);
      XSetFont(xpisgc.dpy,xpisgc.invgc,xpisgc.xfontstruct->fid);
   }


   /* Create Process Images
    */
   if(iconstyle != CHANGE_NOT) {
      switch(iconstyle) {
	 case I_SMALL:
	    if(small_init == NO) {
	       small_images.zombie = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)Z_small_bits,
			    Z_small_width, Z_small_height);
	       small_images.runner = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)R_small_bits,
			    R_small_width, R_small_height);
	       small_images.stopper= XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)T_small_bits,
			    T_small_width, T_small_height);
	       small_images.sleeper= XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)S_small_bits,
			    S_small_width, S_small_height);
	       small_images.waiter = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)W_small_bits,
			    W_small_width, W_small_height);
	       small_images.killer = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)K_small_bits,
			    K_small_width, K_small_height);
	       small_images.ImageHeight = Z_small_height;
	       small_images.ImageWidth = Z_small_width;
	       small_init = YES;
	    }
	    xpisgc.images = &small_images;
	    break;

	 case I_REGULAR:
	    if(reg_init == NO) {
	       reg_images.zombie = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)Z_medium_bits,
			    Z_medium_width, Z_medium_height);
	       reg_images.runner = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)R_medium_bits,
			    R_medium_width, R_medium_height);
	       reg_images.stopper= XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)T_medium_bits,
			    T_medium_width, T_medium_height);
	       reg_images.sleeper= XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)S_medium_bits,
			    S_medium_width, S_medium_height);
	       reg_images.waiter = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)W_medium_bits,
			    W_medium_width, W_medium_height);
	       reg_images.killer = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)K_medium_bits,
			    K_medium_width, K_medium_height);
	       reg_images.ImageHeight = Z_medium_height;
	       reg_images.ImageWidth = Z_medium_width;
	       reg_init = YES;
	    }
	    xpisgc.images = &reg_images;
	    break;

	 case I_FUN:
	    if(fun_init == NO) {
	       fun_images.zombie = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)Z_fun_bits,
			    Z_fun_width, Z_fun_height);
	       fun_images.runner = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)R_fun_bits,
			    R_fun_width, R_fun_height);
	       fun_images.stopper = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)T_fun_bits,
			    T_fun_width, T_fun_height);
	       fun_images.sleeper = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)S_fun_bits,
			    S_fun_width, S_fun_height);
	       fun_images.waiter = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)W_fun_bits,
			    W_fun_width, W_fun_height);
	       fun_images.killer = XCreateBitmapFromData(xpisgc.dpy,
			    (Drawable)xpisgc.xwin,(char *)K_fun_bits,
			    K_fun_width, K_fun_height);
	       fun_images.ImageWidth = Z_fun_width;
	       fun_images.ImageHeight = Z_fun_height;
	       fun_init = YES;
	    }
	    xpisgc.images = &fun_images;
	    break;
      }

      xpisgc.IconHeight = xpisgc.images->ImageHeight + 
			  xpisgc.xfontstruct->descent + 
			  xpisgc.xfontstruct->ascent;
      xpisgc.VerSpareSpace = xpisgc.images->ImageHeight;
      xpisgc.HorSpareSpace = xpisgc.images->ImageWidth / 3;
   }

   /* Set tree or list 
    */
   if(tree != CHANGE_NOT) {
      if((tree != YES) && (tree != NO))
	 puterr("initgc: variable tree must be yes or no at function call");
      else
	 xpisgc.tree = tree;
   }

   return(&xpisgc);
}


